# Copyright 2021 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from oslo_utils import units
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.scenario import manager

CONF = config.CONF


class ImageQuotaTest(manager.ScenarioTest):
    credentials = ['primary', 'system_admin']

    @classmethod
    def skip_checks(cls):
        super(ImageQuotaTest, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(ImageQuotaTest, cls).resource_setup()

        # Figure out and record the glance service id
        services = cls.os_system_admin.identity_services_v3_client.\
            list_services()
        glance_services = [x for x in services['services']
                           if x['name'] == 'glance']
        cls.glance_service_id = glance_services[0]['id']

        # Pre-create all the quota limits and record their IDs so we can
        # update them in-place without needing to know which ones have been
        # created and in which order.
        cls.limit_ids = {}

        try:
            cls.limit_ids['image_size_total'] = cls._create_limit(
                'image_size_total', 10)
            cls.limit_ids['image_stage_total'] = cls._create_limit(
                'image_stage_total', 10)
            cls.limit_ids['image_count_total'] = cls._create_limit(
                'image_count_total', 10)
            cls.limit_ids['image_count_uploading'] = cls._create_limit(
                'image_count_uploading', 10)
        except lib_exc.Forbidden:
            # If we fail to set limits, it means they are not
            # registered, and thus we will skip these tests once we
            # have our os_system_admin client and run
            # check_quotas_enabled().
            pass

    def setUp(self):
        super(ImageQuotaTest, self).setUp()
        self.created_images = []

    def create_image(self, data=None, **kwargs):
        """Wrapper that returns a test image."""

        if 'name' not in kwargs:
            name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__name__ + "-image")
            kwargs['name'] = name

        params = dict(kwargs)
        if data:
            # NOTE: On glance v1 API, the data should be passed on
            # a header. Then here handles the data separately.
            params['data'] = data

        image = self.image_client.create_image(**params)
        # Image objects returned by the v1 client have the image
        # data inside a dict that is keyed against 'image'.
        if 'image' in image:
            image = image['image']
        self.created_images.append(image['id'])
        self.addCleanup(
            self.image_client.wait_for_resource_deletion,
            image['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.image_client.delete_image, image['id'])
        return image

    def check_quotas_enabled(self):
        # Check to see if we should even be running these tests. Use
        # the presence of a registered limit that we recognize as an
        # indication.  This will be set up by the operator (or
        # devstack) if glance is configured to use/honor the unified
        # limits. If one is set, they must all be set, because glance
        # has a single all-or-nothing flag for whether or not to use
        # keystone limits. If anything, checking only one helps to
        # assert the assumption that, if enabled, they must all be at
        # least registered for proper operation.
        registered_limits = self.os_system_admin.identity_limits_client.\
            get_registered_limits()['registered_limits']
        if 'image_count_total' not in [x['resource_name']
                                       for x in registered_limits]:
            raise self.skipException('Target system is not configured with '
                                     'glance unified limits')

    @classmethod
    def _create_limit(cls, name, value):
        return cls.os_system_admin.identity_limits_client.create_limit(
            CONF.identity.region, cls.glance_service_id,
            cls.image_client.tenant_id, name, value)['limits'][0]['id']

    def _update_limit(self, name, value):
        self.os_system_admin.identity_limits_client.update_limit(
            self.limit_ids[name], value)

    def _cleanup_images(self):
        while self.created_images:
            image_id = self.created_images.pop()
            try:
                self.image_client.delete_image(image_id)
            except lib_exc.NotFound:
                pass

    @decorators.idempotent_id('9b74fe24-183b-41e6-bf42-84c2958a7be8')
    @utils.services('image', 'identity')
    def test_image_count_quota(self):
        self.check_quotas_enabled()

        # Set a quota on the number of images for our tenant to one.
        self._update_limit('image_count_total', 1)

        # Create one image
        image = self.create_image(name='first',
                                  container_format='bare',
                                  disk_format='raw',
                                  visibility='private')

        # Second image would put us over quota, so expect failure.
        self.assertRaises(lib_exc.OverLimit,
                          self.create_image,
                          name='second',
                          container_format='bare',
                          disk_format='raw',
                          visibility='private')

        # Update our limit to two.
        self._update_limit('image_count_total', 2)

        # Now the same create should succeed.
        self.create_image(name='second',
                          container_format='bare',
                          disk_format='raw',
                          visibility='private')

        # Third image would put us over quota, so expect failure.
        self.assertRaises(lib_exc.OverLimit,
                          self.create_image,
                          name='third',
                          container_format='bare',
                          disk_format='raw',
                          visibility='private')

        # Delete the first image to put us under quota.
        self.image_client.delete_image(image['id'])

        # Now the same create should succeed.
        self.create_image(name='third',
                          container_format='bare',
                          disk_format='raw',
                          visibility='private')

        # Delete all the images we created before the next test runs,
        # so that it starts with full quota.
        self._cleanup_images()

    @decorators.idempotent_id('b103788b-5329-4aa9-8b0d-97f8733460db')
    @utils.services('image', 'identity')
    def test_image_count_uploading_quota(self):
        if not CONF.image_feature_enabled.import_image:
            skip_msg = (
                "%s skipped as image import is not available" % __name__)
            raise self.skipException(skip_msg)

        self.check_quotas_enabled()

        # Set a quota on the number of images we can have in uploading state.
        self._update_limit('image_stage_total', 10)
        self._update_limit('image_size_total', 10)
        self._update_limit('image_count_total', 10)
        self._update_limit('image_count_uploading', 1)

        file_content = data_utils.random_bytes(1 * units.Mi)

        # Create and stage an image
        image1 = self.create_image(name='first',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.image_client.stage_image_file(image1['id'],
                                           io.BytesIO(file_content))

        # Check that we can not stage another
        image2 = self.create_image(name='second',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.assertRaises(lib_exc.OverLimit,
                          self.image_client.stage_image_file,
                          image2['id'], io.BytesIO(file_content))

        # ... nor upload directly
        image3 = self.create_image(name='third',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.assertRaises(lib_exc.OverLimit,
                          self.image_client.store_image_file,
                          image3['id'],
                          io.BytesIO(file_content))

        # Update our quota to make room
        self._update_limit('image_count_uploading', 2)

        # Now our upload should work
        self.image_client.store_image_file(image3['id'],
                                           io.BytesIO(file_content))

        # ...and because that is no longer in uploading state, we should be
        # able to stage our second image from above.
        self.image_client.stage_image_file(image2['id'],
                                           io.BytesIO(file_content))

        # Finish our import of image2
        self.image_client.image_import(image2['id'], method='glance-direct')
        waiters.wait_for_image_imported_to_stores(self.image_client,
                                                  image2['id'])

        # Set our quota back to one
        self._update_limit('image_count_uploading', 1)

        # Since image1 is still staged, we should not be able to upload
        # an image.
        image4 = self.create_image(name='fourth',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.assertRaises(lib_exc.OverLimit,
                          self.image_client.store_image_file,
                          image4['id'],
                          io.BytesIO(file_content))

        # Finish our import of image1 to make space in our uploading quota.
        self.image_client.image_import(image1['id'], method='glance-direct')
        waiters.wait_for_image_imported_to_stores(self.image_client,
                                                  image1['id'])

        # Make sure that freed up the one upload quota to complete our upload
        self.image_client.store_image_file(image4['id'],
                                           io.BytesIO(file_content))

        # Delete all the images we created before the next test runs,
        # so that it starts with full quota.
        self._cleanup_images()

    @decorators.idempotent_id('05e8d064-c39a-4801-8c6a-465df375ec5b')
    @utils.services('image', 'identity')
    def test_image_size_quota(self):
        self.check_quotas_enabled()

        # Set a quota on the image size for our tenant to 1MiB, and allow ten
        # images.
        self._update_limit('image_size_total', 1)
        self._update_limit('image_count_total', 10)
        self._update_limit('image_count_uploading', 10)

        file_content = data_utils.random_bytes(1 * units.Mi)

        # Create and upload a 1MiB image.
        image1 = self.create_image(name='first',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.image_client.store_image_file(image1['id'],
                                           io.BytesIO(file_content))

        # Create and upload a second 1MiB image. This succeeds, but
        # after completion, we are over quota. Despite us being at
        # quota above, the initial quota check for the second
        # operation has no idea what the image size will be, and thus
        # uses delta=0. This will succeed because we're not
        # technically over-quota and have not asked for any more (this
        # is oslo.limit behavior). After the second operation,
        # however, we will be over-quota regardless of the delta and
        # subsequent attempts will fail. Because glance goes not
        # require an image size to be declared before upload, this is
        # really the best it can do without an API change.
        image2 = self.create_image(name='second',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.image_client.store_image_file(image2['id'],
                                           io.BytesIO(file_content))

        # Create and attempt to upload a third 1MiB image. This should fail to
        # upload (but not create) because we are over quota.
        image3 = self.create_image(name='third',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.assertRaises(lib_exc.OverLimit,
                          self.image_client.store_image_file,
                          image3['id'], io.BytesIO(file_content))

        # Increase our size quota to 2MiB.
        self._update_limit('image_size_total', 2)

        # Now the upload of the already-created image is allowed, but
        # after completion, we are over quota again.
        self.image_client.store_image_file(image3['id'],
                                           io.BytesIO(file_content))

        # Create and attempt to upload a fourth 1MiB image. This should
        # fail to upload (but not create) because we are over quota.
        image4 = self.create_image(name='fourth',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.assertRaises(lib_exc.OverLimit,
                          self.image_client.store_image_file,
                          image4['id'], io.BytesIO(file_content))

        # Delete our first image to make space in our existing 2MiB quota.
        self.image_client.delete_image(image1['id'])

        # Now the upload of the already-created image is allowed.
        self.image_client.store_image_file(image4['id'],
                                           io.BytesIO(file_content))

        # Delete all the images we created before the next test runs,
        # so that it starts with full quota.
        self._cleanup_images()

    @decorators.idempotent_id('fc76b8d9-aae5-46fb-9285-099e37f311f7')
    @utils.services('image', 'identity')
    def test_image_stage_quota(self):
        if not CONF.image_feature_enabled.import_image:
            skip_msg = (
                "%s skipped as image import is not available" % __name__)
            raise self.skipException(skip_msg)

        self.check_quotas_enabled()

        # Create a staging quota of 1MiB, allow 10MiB of active
        # images, and a total of ten images.
        self._update_limit('image_stage_total', 1)
        self._update_limit('image_size_total', 10)
        self._update_limit('image_count_total', 10)
        self._update_limit('image_count_uploading', 10)

        file_content = data_utils.random_bytes(1 * units.Mi)

        # Create and stage a 1MiB image.
        image1 = self.create_image(name='first',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.image_client.stage_image_file(image1['id'],
                                           io.BytesIO(file_content))

        # Create and stage a second 1MiB image. This succeeds, but
        # after completion, we are over quota.
        image2 = self.create_image(name='second',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.image_client.stage_image_file(image2['id'],
                                           io.BytesIO(file_content))

        # Create and attempt to stage a third 1MiB image. This should fail to
        # stage (but not create) because we are over quota.
        image3 = self.create_image(name='third',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.assertRaises(lib_exc.OverLimit,
                          self.image_client.stage_image_file,
                          image3['id'], io.BytesIO(file_content))

        # Make sure that even though we are over our stage quota, we
        # can still create and upload an image the regular way.
        image_upload = self.create_image(name='uploaded',
                                         container_format='bare',
                                         disk_format='raw',
                                         visibility='private')
        self.image_client.store_image_file(image_upload['id'],
                                           io.BytesIO(file_content))

        # Increase our stage quota to two MiB.
        self._update_limit('image_stage_total', 2)

        # Now the upload of the already-created image is allowed, but
        # after completion, we are over quota again.
        self.image_client.stage_image_file(image3['id'],
                                           io.BytesIO(file_content))

        # Create and attempt to stage a fourth 1MiB image. This should
        # fail to stage (but not create) because we are over quota.
        image4 = self.create_image(name='fourth',
                                   container_format='bare',
                                   disk_format='raw',
                                   visibility='private')
        self.assertRaises(lib_exc.OverLimit,
                          self.image_client.stage_image_file,
                          image4['id'], io.BytesIO(file_content))

        # Finish our import of image1 to make space in our stage quota.
        self.image_client.image_import(image1['id'], method='glance-direct')
        waiters.wait_for_image_imported_to_stores(self.image_client,
                                                  image1['id'])

        # Now the upload of the already-created image is allowed.
        self.image_client.stage_image_file(image4['id'],
                                           io.BytesIO(file_content))

        # Delete all the images we created before the next test runs,
        # so that it starts with full quota.
        self._cleanup_images()



# Copyright 2015 Fujitsu(fnst) Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import snapshots as schema
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import base_compute_client


class SnapshotsClient(base_compute_client.BaseComputeClient):

    def create_snapshot(self, volume_id, **kwargs):
        """Create a snapshot.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-snapshot
        """
        post_body = {
            'volume_id': volume_id
        }
        post_body.update(kwargs)
        post_body = json.dumps({'snapshot': post_body})
        resp, body = self.post('os-snapshots', post_body)
        body = json.loads(body)
        self.validate_response(schema.create_get_snapshot, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_snapshot(self, snapshot_id):
        url = "os-snapshots/%s" % snapshot_id
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.create_get_snapshot, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_snapshots(self, detail=False, params=None):
        """List snapshots.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-snapshots
        """
        url = 'os-snapshots'

        if detail:
            url += '/detail'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_snapshots, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_snapshot(self, snapshot_id):
        resp, body = self.delete("os-snapshots/%s" % snapshot_id)
        self.validate_response(schema.delete_snapshot, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_snapshot(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Return the primary type of resource this client works with."""
        return 'snapshot'



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random
import testtools

from tempest.api.object_storage import base
from tempest.common import custom_matchers
from tempest.common import object_storage
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class AccountTest(base.BaseObjectTest):
    """Test account metadata and containers"""

    credentials = [['operator', CONF.object_storage.operator_role],
                   ['operator_alt', CONF.object_storage.operator_role]]
    containers = []

    @classmethod
    def setup_credentials(cls):
        super(AccountTest, cls).setup_credentials()
        cls.os_operator = cls.os_roles_operator_alt

    @classmethod
    def resource_setup(cls):
        super(AccountTest, cls).resource_setup()
        for i in range(ord('a'), ord('f') + 1):
            name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name='%s-' % bytes((i,)))
            cls.container_client.update_container(name)
            cls.addClassResourceCleanup(object_storage.delete_containers,
                                        [name],
                                        cls.container_client,
                                        cls.object_client)
            cls.containers.append(name)
        cls.containers_count = len(cls.containers)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('3499406a-ae53-4f8c-b43a-133d4dc6fe3f')
    def test_list_containers(self):
        """Test listing containers"""
        resp, container_list = self.account_client.list_account_containers()
        self.assertHeaders(resp, 'Account', 'GET')

        self.assertIsNotNone(container_list)

        for container_name in self.containers:
            self.assertIn(str(container_name).encode('utf-8'),
                          container_list)

    @decorators.idempotent_id('884ec421-fbad-4fcc-916b-0580f2699565')
    def test_list_no_containers(self):
        """Test listing containers for an account without container"""

        # To test listing no containers, create new user other than
        # the base user of this instance.
        resp, container_list = \
            self.os_operator.account_client.list_account_containers()

        # When sending a request to an account which has not received a PUT
        # container request, the response does not contain 'accept-ranges'
        # header. This is a special case, therefore the existence of response
        # headers is checked without custom matcher.
        #
        # As the expected response is 204 No Content, Content-Length presence
        # is not checked here intentionally. According to RFC 7230 a server
        # MUST NOT send the header in such responses. Thus, clients should not
        # depend on this header. However, the standard does not require them
        # to validate the server's behavior. We leverage that to not refuse
        # any implementation violating it like Swift [1] or some versions of
        # Ceph RadosGW [2].
        # [1] https://bugs.launchpad.net/swift/+bug/1537811
        # [2] http://tracker.ceph.com/issues/13582
        self.assertIn('x-timestamp', resp)
        self.assertIn('x-account-bytes-used', resp)
        self.assertIn('x-account-container-count', resp)
        self.assertIn('x-account-object-count', resp)
        self.assertIn('content-type', resp)
        self.assertIn('x-trans-id', resp)
        self.assertIn('date', resp)

        # Check only the format of common headers with custom matcher
        self.assertThat(resp, custom_matchers.AreAllWellFormatted())

        self.assertEmpty(container_list)

    @decorators.idempotent_id('1c7efa35-e8a2-4b0b-b5ff-862c7fd83704')
    def test_list_containers_with_format_json(self):
        """Test listing containers setting format parameter to 'json'"""
        params = {'format': 'json'}
        resp, container_list = self.account_client.list_account_containers(
            params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertIsNotNone(container_list)
        self.assertTrue([c['name'] for c in container_list])
        self.assertTrue([c['count'] for c in container_list])
        self.assertTrue([c['bytes'] for c in container_list])

    @decorators.idempotent_id('4477b609-1ca6-4d4b-b25d-ad3f01086089')
    def test_list_containers_with_format_xml(self):
        """Test listing containers setting format parameter to 'xml'"""
        params = {'format': 'xml'}
        resp, container_list = self.account_client.list_account_containers(
            params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertIsNotNone(container_list)
        self.assertEqual(container_list.tag, 'account')
        self.assertIn('name', container_list.keys())
        self.assertEqual(container_list.find(".//container").tag, 'container')
        self.assertEqual(container_list.find(".//name").tag, 'name')
        self.assertEqual(container_list.find(".//count").tag, 'count')
        self.assertEqual(container_list.find(".//bytes").tag, 'bytes')

    @decorators.idempotent_id('6eb04a6a-4860-4e31-ba91-ea3347d76b58')
    @testtools.skipIf(
        not CONF.object_storage_feature_enabled.discoverability,
        'Discoverability function is disabled')
    def test_list_extensions(self):
        """Test listing capabilities"""
        resp = self.capabilities_client.list_capabilities()

        self.assertThat(resp, custom_matchers.AreAllWellFormatted())

    @decorators.idempotent_id('5cfa4ab2-4373-48dd-a41f-a532b12b08b2')
    def test_list_containers_with_limit(self):
        """Test listing containers with limit parameter

        Listing containers limited to one of them, half of them, and then all
        of them.
        """
        for limit in (1, self.containers_count // 2,
                      self.containers_count):
            params = {'limit': limit}
            resp, container_list = \
                self.account_client.list_account_containers(params=params)
            self.assertHeaders(resp, 'Account', 'GET')

            self.assertEqual(len(container_list), limit)

    @decorators.idempotent_id('638f876d-6a43-482a-bbb3-0840bca101c6')
    def test_list_containers_with_marker(self):
        """Test listing containers with marker parameter

        First expect to get 0 container as we specified the last container
        as marker, second expect to get the bottom half of the containers.
        """
        params = {'marker': self.containers[-1]}
        resp, container_list = \
            self.account_client.list_account_containers(params=params)
        self.assertHeaders(resp, 'Account', 'GET')

        self.assertEmpty(container_list)

        params = {'marker': self.containers[self.containers_count // 2]}
        resp, container_list = \
            self.account_client.list_account_containers(params=params)
        self.assertHeaders(resp, 'Account', 'GET')

        self.assertEqual(len(container_list),
                         self.containers_count // 2 - 1)

    @decorators.idempotent_id('5ca164e4-7bde-43fa-bafb-913b53b9e786')
    def test_list_containers_with_end_marker(self):
        """Test listing containers with end_marker parameter

        First expect to get 0 container as we specified first container as
        end_marker, second expect to get the top half of the containers
        """
        params = {'end_marker': self.containers[0]}
        resp, container_list = \
            self.account_client.list_account_containers(params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertEmpty(container_list)

        params = {'end_marker': self.containers[self.containers_count // 2]}
        resp, container_list = \
            self.account_client.list_account_containers(params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertEqual(len(container_list), self.containers_count // 2)

    @decorators.idempotent_id('ac8502c2-d4e4-4f68-85a6-40befea2ef5e')
    def test_list_containers_with_marker_and_end_marker(self):
        """Test listing containers with marker and end_marker parameter

        If we use the first container as marker, and the last container as
        end_marker, then we should get all containers excluding the first one
        and the last one.
        """
        params = {'marker': self.containers[0],
                  'end_marker': self.containers[self.containers_count - 1]}
        resp, container_list = self.account_client.list_account_containers(
            params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertEqual(len(container_list), self.containers_count - 2)

    @decorators.idempotent_id('f7064ae8-dbcc-48da-b594-82feef6ea5af')
    def test_list_containers_with_limit_and_marker(self):
        """Test listing containers combining marker and limit parameter

        Result are always limited by the limit whatever the marker.
        """
        for marker in random.choice(self.containers):
            limit = random.randint(0, self.containers_count - 1)
            params = {'marker': marker,
                      'limit': limit}
            resp, container_list = \
                self.account_client.list_account_containers(params=params)
            self.assertHeaders(resp, 'Account', 'GET')

            self.assertLessEqual(len(container_list), limit,
                                 str(container_list))

    @decorators.idempotent_id('888a3f0e-7214-4806-8e50-5e0c9a69bb5e')
    def test_list_containers_with_limit_and_end_marker(self):
        """Test listing containers combining end_marker and limit parameter

        Result are always limited by the limit whatever the end_marker.
        """
        # list containers combining limit and end_marker param
        limit = random.randint(1, self.containers_count)
        params = {'limit': limit,
                  'end_marker': self.containers[self.containers_count // 2]}
        resp, container_list = self.account_client.list_account_containers(
            params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertEqual(len(container_list),
                         min(limit, self.containers_count // 2))

    @decorators.idempotent_id('8cf98d9c-e3a0-4e44-971b-c87656fdddbd')
    def test_list_containers_with_limit_and_marker_and_end_marker(self):
        """Test listing containers combining marker and end_marker and limit

        Result are always limited by the limit whatever the marker and the
        end_marker.
        """
        limit = random.randint(1, self.containers_count)
        params = {'limit': limit,
                  'marker': self.containers[0],
                  'end_marker': self.containers[self.containers_count - 1]}
        resp, container_list = self.account_client.list_account_containers(
            params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertEqual(len(container_list),
                         min(limit, self.containers_count - 2))

    @decorators.idempotent_id('365e6fc7-1cfe-463b-a37c-8bd08d47b6aa')
    def test_list_containers_with_prefix(self):
        """Test listing containers that have a name starting with a prefix"""
        prefix = 'tempest-a'
        params = {'prefix': prefix}
        resp, container_list = self.account_client.list_account_containers(
            params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        for container in container_list:
            self.assertEqual(True, container.decode(
                'utf-8').startswith(prefix))

    @decorators.idempotent_id('b1811cff-d1ed-4c15-a52e-efd8de41cf34')
    def test_list_containers_reverse_order(self):
        """Test listing containers in reverse order"""
        _, orig_container_list = self.account_client.list_account_containers()

        params = {'reverse': True}
        resp, container_list = self.account_client.list_account_containers(
            params=params)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertEqual(sorted(orig_container_list, reverse=True),
                         container_list)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('4894c312-6056-4587-8d6f-86ffbf861f80')
    def test_list_account_metadata(self):
        """Test listing account metadata"""
        # set metadata to account
        metadata = {'test-account-meta1': 'Meta1',
                    'test-account-meta2': 'Meta2'}
        resp, _ = self.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=metadata)

        resp, _ = self.account_client.list_account_metadata()
        self.assertHeaders(resp, 'Account', 'HEAD')
        self.assertIn('x-account-meta-test-account-meta1', resp)
        self.assertIn('x-account-meta-test-account-meta2', resp)
        self.account_client.create_update_or_delete_account_metadata(
            delete_metadata=metadata)

    @decorators.idempotent_id('b904c2e3-24c2-4dba-ad7d-04e90a761be5')
    def test_list_no_account_metadata(self):
        """Test listing account metadata for account without metadata"""
        resp, _ = self.account_client.list_account_metadata()
        self.assertHeaders(resp, 'Account', 'HEAD')
        self.assertNotIn('x-account-meta-', str(resp))

    @decorators.idempotent_id('e2a08b5f-3115-4768-a3ee-d4287acd6c08')
    def test_update_account_metadata_with_create_metadata(self):
        """Test adding metadata to account"""
        metadata = {'test-account-meta1': 'Meta1'}
        resp, _ = self.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=metadata)
        self.assertHeaders(resp, 'Account', 'POST')

        resp, _ = self.account_client.list_account_metadata()
        self.assertIn('x-account-meta-test-account-meta1', resp)
        self.assertEqual(resp['x-account-meta-test-account-meta1'],
                         metadata['test-account-meta1'])

        self.account_client.create_update_or_delete_account_metadata(
            delete_metadata=metadata)

    @decorators.idempotent_id('9f60348d-c46f-4465-ae06-d51dbd470953')
    def test_update_account_metadata_with_delete_metadata(self):
        """Test deleting metadata from account"""
        metadata = {'test-account-meta1': 'Meta1'}
        self.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=metadata)
        resp, _ = self.account_client.create_update_or_delete_account_metadata(
            delete_metadata=metadata)
        self.assertHeaders(resp, 'Account', 'POST')

        resp, _ = self.account_client.list_account_metadata()
        self.assertNotIn('x-account-meta-test-account-meta1', resp)

    @decorators.idempotent_id('64fd53f3-adbd-4639-af54-436e4982dbfb')
    def test_update_account_metadata_with_create_metadata_key(self):
        """Test adding metadata to account with empty value

        Adding metadata with empty value to account, the metadata is not
        registered.
        """
        metadata = {'test-account-meta1': ''}
        resp, _ = self.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=metadata)
        self.assertHeaders(resp, 'Account', 'POST')

        resp, _ = self.account_client.list_account_metadata()
        self.assertNotIn('x-account-meta-test-account-meta1', resp)

    @decorators.idempotent_id('d4d884d3-4696-4b85-bc98-4f57c4dd2bf1')
    def test_update_account_metadata_with_delete_metadata_key(self):
        """Test deleting metadata from account with empty value

        Although the value of metadata is not set, the feature of deleting
        metadata is valid, so the metadata is removed from account.
        """
        metadata_1 = {'test-account-meta1': 'Meta1'}
        self.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=metadata_1)
        metadata_2 = {'test-account-meta1': ''}
        resp, _ = self.account_client.create_update_or_delete_account_metadata(
            delete_metadata=metadata_2)
        self.assertHeaders(resp, 'Account', 'POST')

        resp, _ = self.account_client.list_account_metadata()
        self.assertNotIn('x-account-meta-test-account-meta1', resp)

    @decorators.idempotent_id('8e5fc073-59b9-42ee-984a-29ed11b2c749')
    def test_update_account_metadata_with_create_and_delete_metadata(self):
        """Test adding and deleting metadata simultaneously

        Send a request adding and deleting metadata requests simultaneously,
        both adding and deleting of metadata will succeed.
        """
        metadata_1 = {'test-account-meta1': 'Meta1'}
        self.account_client.create_update_or_delete_account_metadata(
            create_update_metadata=metadata_1)
        metadata_2 = {'test-account-meta2': 'Meta2'}
        resp, _ = (
            self.account_client.create_update_or_delete_account_metadata(
                create_update_metadata=metadata_2,
                delete_metadata=metadata_1))
        self.assertHeaders(resp, 'Account', 'POST')

        resp, _ = self.account_client.list_account_metadata()
        self.assertNotIn('x-account-meta-test-account-meta1', resp)
        self.assertIn('x-account-meta-test-account-meta2', resp)
        self.assertEqual(resp['x-account-meta-test-account-meta2'],
                         metadata_2['test-account-meta2'])

        self.account_client.create_update_or_delete_account_metadata(
            delete_metadata=metadata_2)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import matchers

from tempest.api.volume import base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class VolumesBackupsTest(base.BaseVolumeTest):
    """Test volumes backup"""

    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(VolumesBackupsTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    def restore_backup(self, backup_id):
        # Restore a backup
        restored_volume = self.backups_client.restore_backup(
            backup_id)['restore']

        # Delete backup
        self.addCleanup(self.delete_volume, self.volumes_client,
                        restored_volume['volume_id'])
        self.assertEqual(backup_id, restored_volume['backup_id'])
        waiters.wait_for_volume_resource_status(self.backups_client,
                                                backup_id, 'available')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                restored_volume['volume_id'],
                                                'available')
        return restored_volume

    @decorators.idempotent_id('a66eb488-8ee1-47d4-8e9f-575a095728c6')
    def test_volume_backup_create_get_detailed_list_restore_delete(self):
        """Test create/get/list/restore/delete volume backup

        1. Create volume1 with metadata
        2. Create backup1 from volume1
        3. Show backup1
        4. List backups with detail
        5. Restore backup1
        6. Verify backup1 has been restored successfully with the metadata
           of volume1
        """
        # Create a volume with metadata
        metadata = {"vol-meta1": "value1",
                    "vol-meta2": "value2",
                    "vol-meta3": "value3"}
        volume = self.create_volume(metadata=metadata)
        self.addCleanup(self.delete_volume, self.volumes_client, volume['id'])

        # Create a backup
        kwargs = {}
        kwargs["name"] = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Backup')
        kwargs["description"] = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="backup-description")
        if CONF.volume.backup_driver == "swift":
            kwargs["container"] = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__class__.__name__ + '-backup-container').lower()
        backup = self.create_backup(volume_id=volume['id'], **kwargs)
        self.assertEqual(kwargs["name"], backup['name'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

        # Get a given backup
        backup = self.backups_client.show_backup(backup['id'])['backup']
        self.assertEqual(kwargs["name"], backup['name'])
        self.assertEqual(kwargs["description"], backup['description'])
        if CONF.volume.backup_driver == "swift":
            self.assertEqual(kwargs["container"], backup['container'])

        # Get all backups with detail
        backups = self.backups_client.list_backups(detail=True)['backups']
        self.assertIn((backup['name'], backup['id']),
                      [(m['name'], m['id']) for m in backups])

        restored_volume = self.restore_backup(backup['id'])

        restored_volume_metadata = self.volumes_client.show_volume(
            restored_volume['volume_id'])['volume']['metadata']

        # Verify the backup has been restored successfully
        # with the metadata of the source volume.
        self.assertThat(restored_volume_metadata.items(),
                        matchers.ContainsAll(metadata.items()))

    @decorators.idempotent_id('07af8f6d-80af-44c9-a5dc-c8427b1b62e6')
    @utils.services('compute')
    def test_backup_create_attached_volume(self):
        """Test backup create using force flag.

        Cinder allows to create a volume backup, whether the volume status
        is "available" or "in-use".
        """
        # Create a server
        volume = self.create_volume(wait_until=False)
        self.addCleanup(self.delete_volume, self.volumes_client, volume['id'])
        validation_resources = self.get_test_validation_resources(
            self.os_primary)
        server = self.create_server(wait_until='SSHABLE',
                                    validation_resources=validation_resources,
                                    validatable=True)
        # Attach volume to instance
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        self.attach_volume(server['id'], volume['id'])
        # Create backup using force flag
        backup_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Backup')
        backup = self.create_backup(volume_id=volume['id'],
                                    name=backup_name, force=True)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'in-use')
        self.assertEqual(backup_name, backup['name'])

    @decorators.idempotent_id('2a8ba340-dff2-4511-9db7-646f07156b15')
    @utils.services('image')
    def test_bootable_volume_backup_and_restore(self):
        """Test backuping and restoring a bootable volume

        1. Create volume1 from image
        2. Create backup1 from volume1
        3. Restore backup1
        4. Verify the restored backup volume is bootable
        """
        # Create volume from image
        img_uuid = CONF.compute.image_ref
        volume = self.create_volume(imageRef=img_uuid)

        volume_details = self.volumes_client.show_volume(
            volume['id'])['volume']
        self.assertTrue(volume_details['bootable'])

        # Create a backup
        backup = self.create_backup(volume_id=volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

        # Restore the backup
        restored_volume_id = self.restore_backup(backup['id'])['volume_id']

        # Verify the restored backup volume is bootable
        restored_volume_info = self.volumes_client.show_volume(
            restored_volume_id)['volume']

        self.assertTrue(restored_volume_info['bootable'])

    @decorators.idempotent_id('f86eff09-2a6d-43c1-905e-8079e5754f1e')
    @utils.services('compute')
    @decorators.related_bug('1703011')
    def test_volume_backup_incremental(self):
        """Test create a backup when latest incremental backup is deleted"""
        # Create a volume
        volume = self.create_volume()

        # Create a server
        server = self.create_server(wait_until='SSHABLE')

        # Attach volume to the server
        self.attach_volume(server['id'], volume['id'])

        # Create a backup to the attached volume
        backup1 = self.create_backup(volume['id'], force=True)

        # Validate backup details
        backup_info = self.backups_client.show_backup(backup1['id'])['backup']
        self.assertEqual(False, backup_info['has_dependent_backups'])
        self.assertEqual(False, backup_info['is_incremental'])

        # Create another incremental backup
        backup2 = self.backups_client.create_backup(
            volume_id=volume['id'], incremental=True, force=True)['backup']
        waiters.wait_for_volume_resource_status(self.backups_client,
                                                backup2['id'], 'available')

        # Validate incremental backup details
        backup2_info = self.backups_client.show_backup(backup2['id'])['backup']
        self.assertEqual(True, backup2_info['is_incremental'])
        self.assertEqual(False, backup2_info['has_dependent_backups'])

        # Delete the last incremental backup that was created
        self.backups_client.delete_backup(backup2['id'])
        self.backups_client.wait_for_resource_deletion(backup2['id'])

        # Create another incremental backup
        backup3 = self.create_backup(
            volume_id=volume['id'], incremental=True, force=True)

        # Validate incremental backup details
        backup3_info = self.backups_client.show_backup(backup3['id'])['backup']
        self.assertEqual(True, backup3_info['is_incremental'])
        self.assertEqual(False, backup3_info['has_dependent_backups'])


class VolumesBackupsV39Test(base.BaseVolumeTest):
    """Test volumes backup with volume microversion greater than 3.8"""

    volume_min_microversion = '3.9'
    volume_max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(VolumesBackupsV39Test, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    @decorators.idempotent_id('9b374cbc-be5f-4d37-8848-7efb8a873dcc')
    def test_update_backup(self):
        """Test updating backup's name and description"""
        # Create volume and backup
        volume = self.create_volume()
        backup = self.create_backup(volume_id=volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

        # Update backup and assert response body for update_backup method
        update_kwargs = {
            'name': data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__class__.__name__ + '-Backup'),
            'description': data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name="volume-backup-description")
        }
        update_backup = self.backups_client.update_backup(
            backup['id'], **update_kwargs)['backup']
        self.assertEqual(backup['id'], update_backup['id'])
        self.assertEqual(update_kwargs['name'], update_backup['name'])

        # Assert response body for show_backup method
        retrieved_backup = self.backups_client.show_backup(
            backup['id'])['backup']
        for key in update_kwargs:
            self.assertEqual(update_kwargs[key], retrieved_backup[key])



# Copyright 2018 SUSE Linux GmbH
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
https://docs.openstack.org/api-ref/identity/v3/index.html#application-credentials
"""

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class ApplicationCredentialsClient(rest_client.RestClient):
    api_version = "v3"

    def create_application_credential(self, user_id, **kwargs):
        """Creates an application credential.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#create-application-credential
        """
        post_body = json.dumps({'application_credential': kwargs})
        resp, body = self.post('users/%s/application_credentials' % user_id,
                               post_body)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_application_credential(self, user_id, application_credential_id):
        """Gets details of an application credential.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#show-application-credential-details
        """
        resp, body = self.get('users/%s/application_credentials/%s' %
                              (user_id, application_credential_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_application_credentials(self, user_id, **params):
        """Lists out all of a user's application credentials.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#list-application-credentials
        """
        url = 'users/%s/application_credentials' % user_id
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_application_credential(self, user_id,
                                      application_credential_id):
        """Deletes an application credential.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v3/index.html#delete-application-credential
        """
        resp, body = self.delete('users/%s/application_credentials/%s' %
                                 (user_id, application_credential_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.object_storage import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class AccountQuotasTest(base.BaseObjectTest):
    """Test account quotas"""

    credentials = [['operator', CONF.object_storage.operator_role],
                   ['reseller', CONF.object_storage.reseller_admin_role]]

    @classmethod
    def setup_credentials(cls):
        super(AccountQuotasTest, cls).setup_credentials()
        cls.os_reselleradmin = cls.os_roles_reseller

    @classmethod
    def resource_setup(cls):
        super(AccountQuotasTest, cls).resource_setup()
        cls.container_name = cls.create_container()

        # Retrieve a ResellerAdmin auth data and use it to set a quota
        # on the client's account
        cls.reselleradmin_auth_data = \
            cls.os_reselleradmin.auth_provider.auth_data

    def setUp(self):
        super(AccountQuotasTest, self).setUp()

        # Set the reselleradmin auth in headers for next account_client
        # request
        self.account_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.reselleradmin_auth_data
        )
        # Set a quota of 20 bytes on the user's account before each test
        headers = {"X-Account-Meta-Quota-Bytes": "20"}

        self.os_roles_operator.account_client.request(
            "POST", url="", headers=headers, body="")

    def tearDown(self):
        # Set the reselleradmin auth in headers for next account_client
        # request
        self.account_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=self.reselleradmin_auth_data
        )
        # remove the quota from the container
        headers = {"X-Remove-Account-Meta-Quota-Bytes": "x"}

        self.os_roles_operator.account_client.request(
            "POST", url="", headers=headers, body="")
        super(AccountQuotasTest, self).tearDown()

    @classmethod
    def resource_cleanup(cls):
        cls.delete_containers()
        super(AccountQuotasTest, cls).resource_cleanup()

    @decorators.attr(type="smoke")
    @decorators.idempotent_id('a22ef352-a342-4587-8f47-3bbdb5b039c4')
    @utils.requires_ext(extension='account_quotas', service='object')
    def test_upload_valid_object(self):
        """Test uploading valid object"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name="TestObject")
        data = data_utils.arbitrary_string()
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, data)

        self.assertHeaders(resp, 'Object', 'PUT')

    @decorators.attr(type=["smoke"])
    @decorators.idempotent_id('63f51f9f-5f1d-4fc6-b5be-d454d70949d6')
    @utils.requires_ext(extension='account_quotas', service='object')
    def test_admin_modify_quota(self):
        """Test ResellerAdmin can modify/remove the quota on a user's account

        Using the account client, the test modifies the quota
        successively to:

        * "25": a random value different from the initial quota value.
        * ""  : an empty value, equivalent to the removal of the quota.
        * "20": set the quota to its initial value.
        """
        for quota in ("25", "", "20"):

            self.account_client.auth_provider.set_alt_auth_data(
                request_part='headers',
                auth_data=self.reselleradmin_auth_data
            )
            headers = {"X-Account-Meta-Quota-Bytes": quota}

            resp, _ = self.os_roles_operator.account_client.request(
                "POST", url="", headers=headers, body="")

            self.assertEqual(resp["status"], "204")
            self.assertHeaders(resp, 'Account', 'POST')



# Copyright 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.scenario import manager

CONF = config.CONF


class TestStampPattern(manager.ScenarioTest):
    """The test suite for both snapshoting and attaching of volume

    This test is for snapshotting an instance/volume and attaching the volume
    created from snapshot to the instance booted from snapshot.
    The following is the scenario outline:
    1. Boot an instance "instance1"
    2. Create a volume "volume1"
    3. Attach volume1 to instance1
    4. Create a filesystem on volume1
    5. Mount volume1
    6. Create a file which timestamp is written in volume1
    7. Unmount volume1
    8. Detach volume1 from instance1
    9. Get a snapshot "snapshot_from_volume" of volume1
    10. Get a snapshot "snapshot_from_instance" of instance1
    11. Boot an instance "instance2" from snapshot_from_instance
    12. Create a volume "volume2"  from snapshot_from_volume
    13. Attach volume2 to instance2
    14. Check the existence of a file which created at 6. in volume2
    """

    @classmethod
    def skip_checks(cls):
        super(TestStampPattern, cls).skip_checks()
        if not CONF.service_available.cinder:
            raise cls.skipException("Cinder is not available")
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder volume snapshots are disabled")

    def _attached_volume_name(
            self, disks_list_before_attach, ip_address, private_key):
        ssh = self.get_remote_client(ip_address, private_key=private_key)

        def _wait_for_volume_available_on_system():
            disks_list_after_attach = ssh.list_disks()
            return len(disks_list_after_attach) > len(disks_list_before_attach)

        if not test_utils.call_until_true(_wait_for_volume_available_on_system,
                                          CONF.compute.build_timeout,
                                          CONF.compute.build_interval):
            raise lib_exc.TimeoutException

        disks_list_after_attach = ssh.list_disks()
        volume_name = [item for item in disks_list_after_attach
                       if item not in disks_list_before_attach][0]
        return volume_name

    @decorators.attr(type='slow')
    @decorators.idempotent_id('10fd234a-515c-41e5-b092-8323060598c5')
    @testtools.skipUnless(CONF.compute_feature_enabled.snapshot,
                          'Snapshotting is not available.')
    @testtools.skipUnless(CONF.network.public_network_id,
                          'The public_network_id option must be specified.')
    @utils.services('compute', 'network', 'volume', 'image')
    def test_stamp_pattern(self):
        # prepare for booting an instance
        keypair = self.create_keypair()
        security_group = self.create_security_group()

        # boot an instance and create a timestamp file in it
        volume = self.create_volume(wait_until=None)
        server = self.create_server(
            key_name=keypair['name'],
            security_groups=[{'name': security_group['name']}])

        # create and add floating IP to server1
        ip_for_server = self.get_server_ip(server)

        # Make sure the machine ssh-able before attaching the volume
        linux_client = self.get_remote_client(
            ip_for_server, private_key=keypair['private_key'],
            server=server)
        disks_list_before_attach = linux_client.list_disks()
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        # The volume retrieved on creation has a non-up-to-date status.
        # Retrieval after it becomes active ensures correct details.
        volume = self.volumes_client.show_volume(volume['id'])['volume']

        self.nova_volume_attach(server, volume)
        volume_device_name = self._attached_volume_name(
            disks_list_before_attach, ip_for_server, keypair['private_key'])

        timestamp = self.create_timestamp(ip_for_server,
                                          volume_device_name,
                                          private_key=keypair['private_key'],
                                          server=server)
        self.nova_volume_detach(server, volume)

        # snapshot the volume
        volume_snapshot = self.create_volume_snapshot(volume['id'])

        # snapshot the instance
        snapshot_image = self.create_server_snapshot(server=server)

        # create second volume from the snapshot(volume2)
        volume_from_snapshot = self.create_volume(
            snapshot_id=volume_snapshot['id'], wait_until=None)

        # boot second instance from the snapshot(instance2)
        server_from_snapshot = self.create_server(
            image_id=snapshot_image['id'],
            key_name=keypair['name'],
            security_groups=[{'name': security_group['name']}])

        # create and add floating IP to server_from_snapshot
        ip_for_snapshot = self.get_server_ip(server_from_snapshot)

        # Make sure the machine ssh-able before attaching the volume
        # Just a live machine is responding
        # for device attach/detach as expected
        linux_client = self.get_remote_client(
            ip_for_snapshot, private_key=keypair['private_key'],
            server=server_from_snapshot)
        disks_list_before_attach = linux_client.list_disks()

        # attach volume2 to instance2
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume_from_snapshot['id'],
                                                'available')
        # The volume retrieved on creation has a non-up-to-date status.
        # Retrieval after it becomes active ensures correct details.
        volume_from_snapshot = self.volumes_client.show_volume(
            volume_from_snapshot['id'])['volume']

        self.nova_volume_attach(server_from_snapshot, volume_from_snapshot)
        volume_device_name = self._attached_volume_name(
            disks_list_before_attach, ip_for_snapshot, keypair['private_key'])

        # check the existence of the timestamp file in the volume2
        timestamp2 = self.get_timestamp(ip_for_snapshot,
                                        volume_device_name,
                                        private_key=keypair['private_key'],
                                        server=server_from_snapshot)
        self.assertEqual(timestamp, timestamp2)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.image.v2 import image_members_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestImageMembersClient(base.BaseServiceTest):
    FAKE_CREATE_SHOW_UPDATE_IMAGE_MEMBER = {
        "status": "pending",
        "created_at": "2013-11-26T07:21:21Z",
        "updated_at": "2013-11-26T07:21:21Z",
        "image_id": "0ae74cc5-5147-4239-9ce2-b0c580f7067e",
        "member_id": "8989447062e04a818baf9e073fd04fa7",
        "schema": "/v2/schemas/member"
    }

    FAKE_LIST_IMAGE_MEMBERS = {
        "members": [
            {
                "created_at": "2013-10-07T17:58:03Z",
                "image_id": "dbc999e3-c52f-4200-bedd-3b18fe7f87fe",
                "member_id": "123456789",
                "schema": "/v2/schemas/member",
                "status": "pending",
                "updated_at": "2013-10-07T17:58:03Z"
            },
            {
                "created_at": "2013-10-07T17:58:55Z",
                "image_id": "dbc999e3-c52f-4200-bedd-3b18fe7f87fe",
                "member_id": "987654321",
                "schema": "/v2/schemas/member",
                "status": "accepted",
                "updated_at": "2013-10-08T12:08:55Z"
            }
        ],
        "schema": "/v2/schemas/members"
    }

    def setUp(self):
        super(TestImageMembersClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = image_members_client.ImageMembersClient(fake_auth,
                                                              'image',
                                                              'regionOne')

    def _test_list_image_members(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_image_members,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_IMAGE_MEMBERS,
            bytes_body,
            image_id="dbc999e3-c52f-4200-bedd-3b18fe7f87fe")

    def _test_show_image_member(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_image_member,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CREATE_SHOW_UPDATE_IMAGE_MEMBER,
            bytes_body,
            image_id="0ae74cc5-5147-4239-9ce2-b0c580f7067e",
            member_id="8989447062e04a818baf9e073fd04fa7")

    def _test_create_image_member(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_image_member,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_SHOW_UPDATE_IMAGE_MEMBER,
            bytes_body,
            image_id="0ae74cc5-5147-4239-9ce2-b0c580f7067e",
            member_id="8989447062e04a818baf9e073fd04fa7")

    def _test_update_image_member(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_image_member,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_CREATE_SHOW_UPDATE_IMAGE_MEMBER,
            bytes_body,
            image_id="0ae74cc5-5147-4239-9ce2-b0c580f7067e",
            member_id="8989447062e04a818baf9e073fd04fa7",
            schema="/v2/schemas/member2")

    def test_list_image_members_with_str_body(self):
        self._test_list_image_members()

    def test_list_image_members_with_bytes_body(self):
        self._test_list_image_members(bytes_body=True)

    def test_show_image_member_with_str_body(self):
        self._test_show_image_member()

    def test_show_image_member_with_bytes_body(self):
        self._test_show_image_member(bytes_body=True)

    def test_create_image_member_with_str_body(self):
        self._test_create_image_member()

    def test_create_image_member_with_bytes_body(self):
        self._test_create_image_member(bytes_body=True)

    def test_delete_image_member(self):
        self.check_service_client_function(
            self.client.delete_image_member,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            image_id="0ae74cc5-5147-4239-9ce2-b0c580f7067e",
            member_id="8989447062e04a818baf9e073fd04fa7",
            status=204)

    def test_update_image_member_with_str_body(self):
        self._test_update_image_member()

    def test_update_image_member_with_bytes_body(self):
        self._test_update_image_member(bytes_body=True)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class InheritedRolesClient(rest_client.RestClient):
    api_version = "v3"

    def create_inherited_role_on_domains_user(
            self, domain_id, user_id, role_id):
        """Assigns a role to a user on projects owned by a domain."""
        resp, body = self.put(
            "OS-INHERIT/domains/%s/users/%s/roles/%s/inherited_to_projects"
            % (domain_id, user_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_inherited_role_from_user_on_domain(
            self, domain_id, user_id, role_id):
        """Revokes an inherited project role from a user on a domain."""
        resp, body = self.delete(
            "OS-INHERIT/domains/%s/users/%s/roles/%s/inherited_to_projects"
            % (domain_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_inherited_project_role_for_user_on_domain(
            self, domain_id, user_id):
        """Lists the inherited project roles on a domain for a user."""
        resp, body = self.get(
            "OS-INHERIT/domains/%s/users/%s/roles/inherited_to_projects"
            % (domain_id, user_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def check_user_inherited_project_role_on_domain(
            self, domain_id, user_id, role_id):
        """Checks whether a user has an inherited project role on a domain."""
        resp, _ = self.head(
            "OS-INHERIT/domains/%s/users/%s/roles/%s/inherited_to_projects"
            % (domain_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def create_inherited_role_on_domains_group(
            self, domain_id, group_id, role_id):
        """Assigns a role to a group on projects owned by a domain."""
        resp, body = self.put(
            "OS-INHERIT/domains/%s/groups/%s/roles/%s/inherited_to_projects"
            % (domain_id, group_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_inherited_role_from_group_on_domain(
            self, domain_id, group_id, role_id):
        """Revokes an inherited project role from a group on a domain."""
        resp, body = self.delete(
            "OS-INHERIT/domains/%s/groups/%s/roles/%s/inherited_to_projects"
            % (domain_id, group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_inherited_project_role_for_group_on_domain(
            self, domain_id, group_id):
        """Lists the inherited project roles on a domain for a group."""
        resp, body = self.get(
            "OS-INHERIT/domains/%s/groups/%s/roles/inherited_to_projects"
            % (domain_id, group_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def check_group_inherited_project_role_on_domain(
            self, domain_id, group_id, role_id):
        """Checks whether a group has an inherited project role on a domain."""
        resp, _ = self.head(
            "OS-INHERIT/domains/%s/groups/%s/roles/%s/inherited_to_projects"
            % (domain_id, group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def create_inherited_role_on_projects_user(
            self, project_id, user_id, role_id):
        """Assigns a role to a user on projects in a subtree."""
        resp, body = self.put(
            "OS-INHERIT/projects/%s/users/%s/roles/%s/inherited_to_projects"
            % (project_id, user_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_inherited_role_from_user_on_project(
            self, project_id, user_id, role_id):
        """Revokes an inherited role from a user on a project."""
        resp, body = self.delete(
            "OS-INHERIT/projects/%s/users/%s/roles/%s/inherited_to_projects"
            % (project_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def check_user_has_flag_on_inherited_to_project(
            self, project_id, user_id, role_id):
        """Check if user has an inherited project role on project"""
        resp, _ = self.head(
            "OS-INHERIT/projects/%s/users/%s/roles/%s/inherited_to_projects"
            % (project_id, user_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def create_inherited_role_on_projects_group(
            self, project_id, group_id, role_id):
        """Assigns a role to a group on projects in a subtree."""
        resp, body = self.put(
            "OS-INHERIT/projects/%s/groups/%s/roles/%s/inherited_to_projects"
            % (project_id, group_id, role_id), None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_inherited_role_from_group_on_project(
            self, project_id, group_id, role_id):
        """Revokes an inherited role from a group on a project."""
        resp, body = self.delete(
            "OS-INHERIT/projects/%s/groups/%s/roles/%s/inherited_to_projects"
            % (project_id, group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def check_group_has_flag_on_inherited_to_project(
            self, project_id, group_id, role_id):
        """Check if group has an inherited project role on project"""
        resp, _ = self.head(
            "OS-INHERIT/projects/%s/groups/%s/roles/%s/inherited_to_projects"
            % (project_id, group_id, role_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)



# Copyright 2012 NTT Data
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import quotas as schema
from tempest.lib.api_schema.response.compute.v2_36 import quotas as schemav236
from tempest.lib.api_schema.response.compute.v2_57 import quotas as schemav257
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class QuotasClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.35', 'schema': schema},
        {'min': '2.36', 'max': '2.56', 'schema': schemav236},
        {'min': '2.57', 'max': None, 'schema': schemav257}]

    def show_quota_set(self, tenant_id, user_id=None, detail=False):
        """List the quota set for a tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-a-quota
        https://docs.openstack.org/api-ref/compute/#show-the-detail-of-quota
        """

        params = {}
        url = 'os-quota-sets/%s' % tenant_id
        if detail:
            url += '/detail'
        if user_id:
            params.update({'user_id': user_id})
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        if detail:
            self.validate_response(schema.get_quota_set_details, resp, body)
        else:
            self.validate_response(schema.get_quota_set, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_default_quota_set(self, tenant_id):
        """List the default quota set for a tenant.

        https://docs.openstack.org/api-ref/compute/#list-default-quotas-for-tenant
        """

        url = 'os-quota-sets/%s/defaults' % tenant_id
        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_quota_set, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_quota_set(self, tenant_id, user_id=None, **kwargs):
        """Updates the tenant's quota limits for one or more resources.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-quotas
        """

        post_body = json.dumps({'quota_set': kwargs})

        if user_id:
            resp, body = self.put('os-quota-sets/%s?user_id=%s' %
                                  (tenant_id, user_id), post_body)
        else:
            resp, body = self.put('os-quota-sets/%s' % tenant_id,
                                  post_body)

        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.update_quota_set, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_quota_set(self, tenant_id):
        """Delete the tenant's quota set.

        https://docs.openstack.org/api-ref/compute/#revert-quotas-to-defaults
        """
        resp, body = self.delete('os-quota-sets/%s' % tenant_id)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.delete_quota, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2016 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log

from tempest.api.compute import base
from tempest.common import compute
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_excs

CONF = config.CONF
LOG = log.getLogger(__name__)


# NOTE(mriedem): This is in the admin directory only because it requires
# force_tenant_isolation=True, but doesn't extend BaseV2ComputeAdminTest
# because it doesn't actually use any admin credentials in the tests.
class AutoAllocateNetworkTest(base.BaseV2ComputeTest):
    """Tests auto-allocating networks with the v2.37 microversion.

    These tests rely on Neutron being enabled. Also, the tenant must not have
    any network resources available to it so we can make sure that Nova
    calls to Neutron to automatically allocate the network topology.
    """

    force_tenant_isolation = True

    min_microversion = '2.37'
    max_microversion = 'latest'

    @classmethod
    def skip_checks(cls):
        super(AutoAllocateNetworkTest, cls).skip_checks()
        if not CONF.service_available.neutron:
            raise cls.skipException('Neutron is required')
        if not utils.is_extension_enabled('auto-allocated-topology',
                                          'network'):
            raise cls.skipException(
                'auto-allocated-topology extension is not available')

    @classmethod
    def setup_credentials(cls):
        # Do not create network resources for these tests.
        cls.set_network_resources()
        super(AutoAllocateNetworkTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(AutoAllocateNetworkTest, cls).setup_clients()
        cls.networks_client = cls.os_primary.networks_client
        cls.routers_client = cls.os_primary.routers_client
        cls.subnets_client = cls.os_primary.subnets_client
        cls.ports_client = cls.os_primary.ports_client

    @classmethod
    def resource_setup(cls):
        super(AutoAllocateNetworkTest, cls).resource_setup()
        # Sanity check that there are no networks available to the tenant.
        # This is essentially what Nova does for getting available networks.
        tenant_id = cls.networks_client.tenant_id
        # (1) Retrieve non-public network list owned by the tenant.
        search_opts = {'tenant_id': tenant_id, 'shared': False}
        nets = cls.networks_client.list_networks(
            **search_opts).get('networks', [])
        if nets:
            raise lib_excs.TempestException(
                'Found tenant networks: %s' % nets)
        # (2) Retrieve shared network list.
        search_opts = {'shared': True}
        nets = cls.networks_client.list_networks(
            **search_opts).get('networks', [])
        if nets:
            raise cls.skipException('Found shared networks: %s' % nets)

    @classmethod
    def resource_cleanup(cls):
        """Deletes any auto_allocated_network and it's associated resources."""

        # Find the auto-allocated router for the tenant.
        # This is a bit hacky since we don't have a great way to find the
        # auto-allocated router given the private tenant network we have.
        routers = cls.routers_client.list_routers().get('routers', [])
        if len(routers) > 1:
            # This indicates a race where nova is concurrently calling the
            # neutron auto-allocated-topology API for multiple server builds
            # at the same time (it's called from nova-compute when setting up
            # networking for a server). Neutron will detect duplicates and
            # automatically clean them up, but there is a window where the API
            # can return multiple and we don't have a good way to filter those
            # out right now, so we'll just handle them.
            LOG.info('(%s) Found more than one router for tenant.',
                     test_utils.find_test_caller())

        # Remove any networks, duplicate or otherwise, that these tests
        # created. All such networks will be in the current tenant. Neutron
        # will cleanup duplicate resources automatically, so ignore 404s.
        search_opts = {'tenant_id': cls.networks_client.tenant_id}
        networks = cls.networks_client.list_networks(
            **search_opts).get('networks', [])

        for router in routers:
            # Disassociate the subnets from the router. Because of the race
            # mentioned above the subnets might not be associated with the
            # router so ignore any 404.
            for network in networks:
                for subnet_id in network['subnets']:
                    test_utils.call_and_ignore_notfound_exc(
                        cls.routers_client.remove_router_interface,
                        router['id'], subnet_id=subnet_id)

            # Delete the router.
            cls.routers_client.delete_router(router['id'])

        for network in networks:
            # Get and delete the ports for the given network.
            ports = cls.ports_client.list_ports(
                network_id=network['id']).get('ports', [])
            for port in ports:
                test_utils.call_and_ignore_notfound_exc(
                    cls.ports_client.delete_port, port['id'])

            # Delete the subnets.
            for subnet_id in network['subnets']:
                test_utils.call_and_ignore_notfound_exc(
                    cls.subnets_client.delete_subnet, subnet_id)

            # Delete the network.
            test_utils.call_and_ignore_notfound_exc(
                cls.networks_client.delete_network, network['id'])

        super(AutoAllocateNetworkTest, cls).resource_cleanup()

    @decorators.idempotent_id('5eb7b8fa-9c23-47a2-9d7d-02ed5809dd34')
    def test_server_create_no_allocate(self):
        """Tests that no networking is allocated for the server."""
        # create the server with no networking
        server = self.create_test_server(networks='none', wait_until='ACTIVE')
        # get the server ips
        addresses = self.servers_client.list_addresses(
            server['id'])['addresses']
        # assert that there is no networking
        self.assertEqual({}, addresses)

    @decorators.idempotent_id('2e6cf129-9e28-4e8a-aaaa-045ea826b2a6')
    def test_server_multi_create_auto_allocate(self):
        """Tests that networking is auto-allocated for multiple servers."""

        # Create multiple servers with auto networking to make sure the
        # automatic network allocation is atomic. Using a minimum of three
        # servers is essential for this scenario because:
        #
        # - First request sees no networks for the tenant so it auto-allocates
        #   one from Neutron, let's call that net1.
        # - Second request sees no networks for the tenant so it auto-allocates
        #   one from Neutron. Neutron creates net2 but sees it's a duplicate
        #   so it queues net2 for deletion and returns net1 from the API and
        #   Nova uses that for the second server request.
        # - Third request sees net1 and net2 for the tenant and fails with a
        #   NetworkAmbiguous 400 error.
        _, servers = compute.create_test_server(
            self.os_primary, networks='auto', wait_until='ACTIVE',
            min_count=3)
        for server in servers:
            self.addCleanup(self.delete_server, server['id'])

        server_nets = set()
        for server in servers:
            # get the server ips
            addresses = self.servers_client.list_addresses(
                server['id'])['addresses']
            # assert that there is networking (should only be one)
            self.assertEqual(1, len(addresses))
            server_nets.add(list(addresses.keys())[0])
        # all servers should be on the same network
        self.assertEqual(1, len(server_nets))

        # List the networks for the tenant; we filter on admin_state_up=True
        # because the auto-allocated-topology code in Neutron won't set that
        # to True until the network is ready and is returned from the API.
        # Duplicate networks created from a race should have
        # admin_state_up=False.
        search_opts = {'tenant_id': self.networks_client.tenant_id,
                       'shared': False,
                       'admin_state_up': True}
        nets = self.networks_client.list_networks(
            **search_opts).get('networks', [])
        self.assertEqual(1, len(nets))
        # verify the single private tenant network is the one that the servers
        # are using also
        self.assertIn(nets[0]['name'], server_nets)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log

from tempest import config
from tempest.lib import exceptions as lib_exc

CONF = config.CONF
LOG = log.getLogger(__name__)


def delete_containers(containers, container_client, object_client):
    """Remove containers and all objects in them.

    The containers should be visible from the container_client given.
    Will not throw any error if the containers don't exist.

    :param containers: List of containers(or string of a container)
                       to be deleted
    :param container_client: Client to be used to delete containers
    :param object_client: Client to be used to delete objects
    """
    if isinstance(containers, str):
        containers = [containers]

    for cont in containers:
        try:
            delete_objects(cont, container_client, object_client)
            container_client.delete_container(cont)
            container_client.wait_for_resource_deletion(cont)
        except lib_exc.NotFound:
            LOG.warning(f"Container {cont} wasn't deleted as it wasn't found.")


def delete_objects(container, container_client, object_client):
    """Remove all objects from container.

    Will not throw any error if the objects do not exist

    :param container: Name of the container that contains the objects to be
                      deleted
    :param container_client: Client to be used to list objects in
                             the container
    :param object_client: Client to be used to delete objects
    """
    params = {'limit': 9999, 'format': 'json'}
    _, objlist = container_client.list_container_objects(container, params)

    for obj in objlist:
        try:
            object_client.delete_object(container, obj['name'])
            object_client.wait_for_resource_deletion(obj['name'], container)
        except lib_exc.NotFound:
            LOG.warning(f"Object {obj} wasn't deleted as it wasn't found.")



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.volume.v3_64 import volumes

# Volume microversion 3.65:
# Add 'consumes_quota' attribute in volume details.

common_show_volume = copy.deepcopy(volumes.common_show_volume)
common_show_volume['properties'].update(
    {'consumes_quota': {'type': 'boolean'}})

create_volume = copy.deepcopy(volumes.create_volume)
create_volume['response_body']['properties']['volume']['properties'].update(
    {'consumes_quota': {'type': 'boolean'}})

# copy unchanged volumes schema
attachments = copy.deepcopy(volumes.attachments)
list_volumes_no_detail = copy.deepcopy(volumes.list_volumes_no_detail)
# show_volume refers to common_show_volume
show_volume = copy.deepcopy(volumes.show_volume)
show_volume['response_body']['properties']['volume'] = common_show_volume
# list_volumes_detail refers to latest common_show_volume
list_volumes_detail = copy.deepcopy(common_show_volume)
list_volumes_with_detail = copy.deepcopy(volumes.list_volumes_with_detail)
list_volumes_with_detail['response_body']['properties']['volumes']['items'] \
    = list_volumes_detail
update_volume = copy.deepcopy(volumes.update_volume)
delete_volume = copy.deepcopy(volumes.delete_volume)
show_volume_summary = copy.deepcopy(volumes.show_volume_summary)
attach_volume = copy.deepcopy(volumes.attach_volume)
set_bootable_volume = copy.deepcopy(volumes.set_bootable_volume)
detach_volume = copy.deepcopy(volumes.detach_volume)
reserve_volume = copy.deepcopy(volumes.reserve_volume)
unreserve_volume = copy.deepcopy(volumes.unreserve_volume)
extend_volume = copy.deepcopy(volumes.extend_volume)
reset_volume_status = copy.deepcopy(volumes.reset_volume_status)
update_volume_readonly = copy.deepcopy(volumes.update_volume_readonly)
force_delete_volume = copy.deepcopy(volumes.force_delete_volume)
retype_volume = copy.deepcopy(volumes.retype_volume)
force_detach_volume = copy.deepcopy(volumes.force_detach_volume)
create_volume_metadata = copy.deepcopy(volumes.create_volume_metadata)
show_volume_metadata = copy.deepcopy(volumes.show_volume_metadata)
update_volume_metadata = copy.deepcopy(volumes.update_volume_metadata)
update_volume_metadata_item = copy.deepcopy(
    volumes.update_volume_metadata_item)
update_volume_image_metadata = copy.deepcopy(
    volumes.update_volume_image_metadata)
delete_volume_image_metadata = copy.deepcopy(
    volumes.delete_volume_image_metadata)
unmanage_volume = copy.deepcopy(volumes.unmanage_volume)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils

CONF = config.CONF


class BaseSecurityGroupsTest(base.BaseV2ComputeTest):
    max_microversion = '2.35'

    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(BaseSecurityGroupsTest, cls).skip_checks()
        if not utils.get_service_list()['network']:
            raise cls.skipException("network service not enabled.")

    @staticmethod
    def generate_random_security_group_id():
        if (CONF.service_available.neutron and
            utils.is_extension_enabled('security-group', 'network')):
            return data_utils.rand_uuid()
        else:
            return data_utils.rand_int_id(start=999)



# Copyright 2017 AT&T.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import assisted_volume_snapshots_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestVolumesClient(base.BaseServiceTest):

    FAKE_SNAPSHOT = {
        "id": "bf7b810c-70df-4c64-88a7-8588f7a6739c",
        "volumeId": "59f17c4f-66d4-4271-be40-f200523423a9"
    }

    def setUp(self):
        super(TestVolumesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = assisted_volume_snapshots_client.\
            AssistedVolumeSnapshotsClient(fake_auth, 'compute', 'regionOne')

    def _test_create_assisted_volume_snapshot(self, bytes_body=False):
        kwargs = {"type": "qcow2", "new_file": "fake_name"}
        self.check_service_client_function(
            self.client.create_assisted_volume_snapshot,
            'tempest.lib.common.rest_client.RestClient.post',
            {"snapshot": self.FAKE_SNAPSHOT},
            bytes_body, status=200, volume_id=self.FAKE_SNAPSHOT['volumeId'],
            snapshot_id=self.FAKE_SNAPSHOT['id'], **kwargs)

    def test_create_assisted_volume_snapshot_with_str_body(self):
        self._test_create_assisted_volume_snapshot()

    def test_create_assisted_volume_snapshot_with_byte_body(self):
        self._test_create_assisted_volume_snapshot(bytes_body=True)

    def test_delete_assisted_volume_snapshot(self):
        self.check_service_client_function(
            self.client.delete_assisted_volume_snapshot,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=204, volume_id=self.FAKE_SNAPSHOT['volumeId'],
            snapshot_id=self.FAKE_SNAPSHOT['id'])



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest.common import waiters
from tempest.lib import decorators


class DHCPAgentSchedulersTestJSON(base.BaseAdminNetworkTest):
    """Test network DHCP agent scheduler extension"""

    @classmethod
    def skip_checks(cls):
        super(DHCPAgentSchedulersTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('dhcp_agent_scheduler', 'network'):
            msg = "dhcp_agent_scheduler extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(DHCPAgentSchedulersTestJSON, cls).resource_setup()
        # NOTE(slaweq): In some cases (like default ML2/OVN deployment)
        # extension is enabled but there may not be any DHCP agent available.
        # In such case those tests should be also skipped.
        dhcp_agents = cls.admin_agents_client.list_agents(
            agent_type="DHCP Agent")['agents']
        if not dhcp_agents:
            msg = ("At least one DHCP agent is required to be running in "
                   "the environment for those tests.")
            raise cls.skipException(msg)
        # Create a network and make sure it will be hosted by a
        # dhcp agent: this is done by creating a regular port
        cls.network = cls.create_network()
        cls.create_subnet(cls.network)
        cls.port = cls.create_port(cls.network)

    @decorators.idempotent_id('f164801e-1dd8-4b8b-b5d3-cc3ac77cfaa5')
    def test_dhcp_port_status_active(self):
        dhcp_ports = self.admin_ports_client.list_ports(
            device_owner='network:dhcp',
            network_id=self.network['id'])['ports']
        for dhcp_port in dhcp_ports:
            waiters.wait_for_port_status(
                client=self.admin_ports_client,
                port_id=dhcp_port['id'],
                status='ACTIVE')

    @decorators.idempotent_id('5032b1fe-eb42-4a64-8f3b-6e189d8b5c7d')
    def test_list_dhcp_agent_hosting_network(self):
        """Test Listing DHCP agents hosting a network"""
        self.admin_networks_client.list_dhcp_agents_on_hosting_network(
            self.network['id'])

    @decorators.idempotent_id('30c48f98-e45d-4ffb-841c-b8aad57c7587')
    def test_list_networks_hosted_by_one_dhcp(self):
        """Test Listing networks hosted by a DHCP agent"""
        body = self.admin_networks_client.list_dhcp_agents_on_hosting_network(
            self.network['id'])
        agents = body['agents']
        self.assertNotEmpty(agents, "no dhcp agent")
        agent = agents[0]
        self.assertTrue(self._check_network_in_dhcp_agent(
            self.network['id'], agent))

    def _check_network_in_dhcp_agent(self, network_id, agent):
        network_ids = []
        body = self.admin_agents_client.list_networks_hosted_by_one_dhcp_agent(
            agent['id'])
        networks = body['networks']
        for network in networks:
            network_ids.append(network['id'])
        return network_id in network_ids

    @decorators.idempotent_id('a0856713-6549-470c-a656-e97c8df9a14d')
    def test_add_remove_network_from_dhcp_agent(self):
        """Test adding and removing network from a DHCP agent"""
        # The agent is now bound to the network, we can free the port
        self.ports_client.delete_port(self.port['id'])
        agent = dict()
        agent['agent_type'] = None
        body = self.admin_agents_client.list_agents()
        agents = body['agents']
        for a in agents:
            if a['agent_type'] == 'DHCP agent':
                agent = a
                break
        self.assertEqual(agent['agent_type'], 'DHCP agent', 'Could not find '
                         'DHCP agent in agent list though dhcp_agent_scheduler'
                         ' is enabled.')
        network = self.create_network()
        network_id = network['id']
        if self._check_network_in_dhcp_agent(network_id, agent):
            self._remove_network_from_dhcp_agent(network_id, agent)
            self._add_dhcp_agent_to_network(network_id, agent)
        else:
            self._add_dhcp_agent_to_network(network_id, agent)
            self._remove_network_from_dhcp_agent(network_id, agent)

    def _remove_network_from_dhcp_agent(self, network_id, agent):
        self.admin_agents_client.delete_network_from_dhcp_agent(
            agent_id=agent['id'],
            network_id=network_id)
        self.assertFalse(self._check_network_in_dhcp_agent(
            network_id, agent))

    def _add_dhcp_agent_to_network(self, network_id, agent):
        self.admin_agents_client.add_dhcp_agent_to_network(
            agent['id'], network_id=network_id)
        self.assertTrue(self._check_network_in_dhcp_agent(
            network_id, agent))



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

list_flavors = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'flavors': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'name': {'type': 'string'},
                        'links': parameter_types.links,
                        'id': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    'required': ['name', 'links', 'id']
                }
            },
            'flavors_links': parameter_types.links
        },
        'additionalProperties': False,
        # NOTE(gmann): flavors_links attribute is not necessary
        # to be present always So it is not 'required'.
        'required': ['flavors']
    }
}

common_flavor_info = {
    'type': 'object',
    'properties': {
        'name': {'type': 'string'},
        'links': parameter_types.links,
        'ram': {'type': 'integer'},
        'vcpus': {'type': 'integer'},
        # 'swap' attributes comes as integer value but if it is empty
        # it comes as "". So defining type of as string and integer.
        'swap': {'type': ['integer', 'string']},
        'disk': {'type': 'integer'},
        'id': {'type': 'string'},
        'OS-FLV-DISABLED:disabled': {'type': 'boolean'},
        'os-flavor-access:is_public': {'type': 'boolean'},
        'rxtx_factor': {'type': 'number'},
        'OS-FLV-EXT-DATA:ephemeral': {'type': 'integer'}
    },
    'additionalProperties': False,
    # 'OS-FLV-DISABLED', 'os-flavor-access', 'rxtx_factor' and
    # 'OS-FLV-EXT-DATA' are API extensions, so they are not 'required'.
    'required': ['name', 'links', 'ram', 'vcpus', 'swap', 'disk', 'id']
}

list_flavors_details = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'flavors': {
                'type': 'array',
                'items': common_flavor_info
            },
            # NOTE(gmann): flavors_links attribute is not necessary
            # to be present always so it is not 'required'.
            'flavors_links': parameter_types.links
        },
        'additionalProperties': False,
        'required': ['flavors']
    }
}

create_update_get_flavor_details = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'flavor': common_flavor_info
        },
        'additionalProperties': False,
        'required': ['flavor']
    }
}

delete_flavor = {
    'status_code': [202]
}



# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common import api_version_request
from tempest.lib.common import api_version_utils
from tempest.lib.common import rest_client
from tempest.lib import exceptions

VOLUME_MICROVERSION = None


class BaseClient(rest_client.RestClient):
    """Base volume service clients class to support microversion."""
    api_microversion_header_name = 'Openstack-Api-Version'

    def get_headers(self, accept_type=None, send_type=None):
        headers = super(BaseClient, self).get_headers(
            accept_type=accept_type, send_type=send_type)
        if VOLUME_MICROVERSION:
            headers[self.api_microversion_header_name] = ('volume %s' %
                                                          VOLUME_MICROVERSION)
        return headers

    def request(self, method, url, extra_headers=False, headers=None,
                body=None, chunked=False):

        resp, resp_body = super(BaseClient, self).request(
            method, url, extra_headers, headers, body, chunked)
        if (VOLUME_MICROVERSION and
            VOLUME_MICROVERSION != api_version_utils.LATEST_MICROVERSION):
            api_version_utils.assert_version_header_matches_request(
                self.api_microversion_header_name,
                'volume %s' % VOLUME_MICROVERSION,
                resp)
        return resp, resp_body

    def get_schema(self, schema_versions_info):
        """Get JSON schema

        This method provides the matching schema for requested
        microversion.

        :param schema_versions_info: List of dict which provides schema
                                     information with range of valid versions.

        Example::

         schema_versions_info = [
             {'min': None, 'max': '2.1', 'schema': schemav21},
             {'min': '2.2', 'max': '2.9', 'schema': schemav22},
             {'min': '2.10', 'max': None, 'schema': schemav210}]
        """
        schema = None
        version = api_version_request.APIVersionRequest(VOLUME_MICROVERSION)
        for items in schema_versions_info:
            min_version = api_version_request.APIVersionRequest(items['min'])
            max_version = api_version_request.APIVersionRequest(items['max'])
            # This is case where COMPUTE_MICROVERSION is None, which means
            # request without microversion So select base v2.1 schema.
            if version.is_null() and items['min'] is None:
                schema = items['schema']
                break
            # else select appropriate schema as per COMPUTE_MICROVERSION
            elif version.matches(min_version, max_version):
                schema = items['schema']
                break
        if schema is None:
            raise exceptions.JSONSchemaNotFound(
                version=version.get_string(),
                schema_versions_info=schema_versions_info)
        return schema



# Copyright (C) 2017 Dell Inc. or its subsidiaries.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.volume.v3 import group_snapshots_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestGroupSnapshotsClient(base.BaseServiceTest):
    FAKE_CREATE_GROUP_SNAPSHOT = {
        "group_snapshot": {
            "id": "6f519a48-3183-46cf-a32f-41815f816666",
            "name": "first_group_snapshot",
            "group_type_id": "58737af7-786b-48b7-ab7c-2447e74b0ef4"
        }
    }

    FAKE_INFO_GROUP_SNAPSHOT = {
        "group_snapshot": {
            "id": "0e701ab8-1bec-4b9f-b026-a7ba4af13578",
            "group_id": "49c8c114-0d68-4e89-b8bc-3f5a674d54be",
            "name": "group-snapshot-001",
            "description": "Test group snapshot 1",
            "group_type_id": "0e58433f-d108-4bf3-a22c-34e6b71ef86b",
            "status": "available",
            "created_at": "2017-06-20T03:50:07Z"
        }
    }

    FAKE_LIST_GROUP_SNAPSHOTS = {
        "group_snapshots": [
            {
                "id": "0e701ab8-1bec-4b9f-b026-a7ba4af13578",
                "group_id": "49c8c114-0d68-4e89-b8bc-3f5a674d54be",
                "name": "group-snapshot-001",
                "description": "Test group snapshot 1",
                "group_type_id": "0e58433f-d108-4bf3-a22c-34e6b71ef86b",
                "status": "available",
                "created_at": "2017-06-20T03:50:07Z",
            },
            {
                "id": "e479997c-650b-40a4-9dfe-77655818b0d2",
                "group_id": "49c8c114-0d68-4e89-b8bc-3f5a674d54be",
                "name": "group-snapshot-002",
                "description": "Test group snapshot 2",
                "group_type_id": "0e58433f-d108-4bf3-a22c-34e6b71ef86b",
                "status": "available",
                "created_at": "2017-06-19T01:52:47Z",
            },
            {
                "id": "c5c4769e-213c-40a6-a568-8e797bb691d4",
                "group_id": "49c8c114-0d68-4e89-b8bc-3f5a674d54be",
                "name": "group-snapshot-003",
                "description": "Test group snapshot 3",
                "group_type_id": "0e58433f-d108-4bf3-a22c-34e6b71ef86b",
                "status": "available",
                "created_at": "2017-06-18T06:34:32Z",
            }
        ]
    }

    def setUp(self):
        super(TestGroupSnapshotsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = group_snapshots_client.GroupSnapshotsClient(
            fake_auth, 'volume', 'regionOne')

    def _test_create_group_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_group_snapshot,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_GROUP_SNAPSHOT,
            bytes_body,
            group_id="49c8c114-0d68-4e89-b8bc-3f5a674d54be",
            status=202)

    def _test_show_group_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_group_snapshot,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_INFO_GROUP_SNAPSHOT,
            bytes_body,
            group_snapshot_id="3fbbcccf-d058-4502-8844-6feeffdf4cb5")

    def _test_list_group_snapshots(self, detail=False, bytes_body=False,
                                   mock_args='group_snapshots', **params):
        resp_body = []
        if detail:
            resp_body = self.FAKE_LIST_GROUP_SNAPSHOTS
        else:
            resp_body = {
                'group_snapshots': [{
                    'id': group_snapshot['id'],
                    'name': group_snapshot['name']}
                    for group_snapshot in
                    self.FAKE_LIST_GROUP_SNAPSHOTS['group_snapshots']
                ]
            }
        self.check_service_client_function(
            self.client.list_group_snapshots,
            'tempest.lib.common.rest_client.RestClient.get',
            resp_body,
            to_utf=bytes_body,
            mock_args=[mock_args],
            detail=detail,
            **params)

    def test_create_group_snapshot_with_str_body(self):
        self._test_create_group_snapshot()

    def test_create_group_snapshot_with_bytes_body(self):
        self._test_create_group_snapshot(bytes_body=True)

    def test_show_group_snapshot_with_str_body(self):
        self._test_show_group_snapshot()

    def test_show_group_snapshot_with_bytes_body(self):
        self._test_show_group_snapshot(bytes_body=True)

    def test_list_group_snapshots_with_str_body(self):
        self._test_list_group_snapshots()

    def test_list_group_snapshots_with_bytes_body(self):
        self._test_list_group_snapshots(bytes_body=True)

    def test_list_group_snapshots_with_detail_with_str_body(self):
        mock_args = "group_snapshots/detail"
        self._test_list_group_snapshots(detail=True, mock_args=mock_args)

    def test_list_group_snapshots_with_detail_with_bytes_body(self):
        mock_args = "group_snapshots/detail"
        self._test_list_group_snapshots(detail=True, bytes_body=True,
                                        mock_args=mock_args)

    def test_list_group_snapshots_with_params(self):
        # Run the test separately for each param, to avoid assertion error
        # resulting from randomized params order.
        mock_args = 'group_snapshots?sort_key=name'
        self._test_list_group_snapshots(mock_args=mock_args, sort_key='name')

        mock_args = 'group_snapshots/detail?limit=10'
        self._test_list_group_snapshots(detail=True, bytes_body=True,
                                        mock_args=mock_args, limit=10)

    def test_delete_group_snapshot(self):
        self.check_service_client_function(
            self.client.delete_group_snapshot,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            group_snapshot_id='0e701ab8-1bec-4b9f-b026-a7ba4af13578',
            status=202)

    def test_reset_group_snapshot_status(self):
        self.check_service_client_function(
            self.client.reset_group_snapshot_status,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            status=202,
            group_snapshot_id='0e701ab8-1bec-4b9f-b026-a7ba4af13578',
            status_to_set='error')



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

get_hypervisor_statistics = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'hypervisor_statistics': {
                'type': 'object',
                'properties': {
                    'count': {'type': 'integer'},
                    'current_workload': {'type': 'integer'},
                    'disk_available_least': {'type': ['integer', 'null']},
                    'free_disk_gb': {'type': 'integer'},
                    'free_ram_mb': {'type': 'integer'},
                    'local_gb': {'type': 'integer'},
                    'local_gb_used': {'type': 'integer'},
                    'memory_mb': {'type': 'integer'},
                    'memory_mb_used': {'type': 'integer'},
                    'running_vms': {'type': 'integer'},
                    'vcpus': {'type': 'integer'},
                    'vcpus_used': {'type': 'integer'}
                },
                'additionalProperties': False,
                'required': ['count', 'current_workload',
                             'disk_available_least', 'free_disk_gb',
                             'free_ram_mb', 'local_gb', 'local_gb_used',
                             'memory_mb', 'memory_mb_used', 'running_vms',
                             'vcpus', 'vcpus_used']
            }
        },
        'additionalProperties': False,
        'required': ['hypervisor_statistics']
    }
}


hypervisor_detail = {
    'type': 'object',
    'properties': {
        'status': {'type': 'string'},
        'state': {'type': 'string'},
        'cpu_info': {'type': 'string'},
        'current_workload': {'type': 'integer'},
        'disk_available_least': {'type': ['integer', 'null']},
        'host_ip': parameter_types.ip_address,
        'free_disk_gb': {'type': 'integer'},
        'free_ram_mb': {'type': 'integer'},
        'hypervisor_hostname': {'type': 'string'},
        'hypervisor_type': {'type': 'string'},
        'hypervisor_version': {'type': 'integer'},
        'id': {'type': ['integer', 'string']},
        'local_gb': {'type': 'integer'},
        'local_gb_used': {'type': 'integer'},
        'memory_mb': {'type': 'integer'},
        'memory_mb_used': {'type': 'integer'},
        'running_vms': {'type': 'integer'},
        'service': {
            'type': 'object',
            'properties': {
                'host': {'type': 'string'},
                'id': {'type': ['integer', 'string']},
                'disabled_reason': {'type': ['string', 'null']}
            },
            'additionalProperties': False,
            'required': ['host', 'id']
        },
        'vcpus': {'type': 'integer'},
        'vcpus_used': {'type': 'integer'}
    },
    'additionalProperties': False,
    # NOTE: When loading os-hypervisor-status extension,
    # a response contains status and state. So these params
    # should not be required.
    'required': ['cpu_info', 'current_workload',
                 'disk_available_least', 'host_ip',
                 'free_disk_gb', 'free_ram_mb',
                 'hypervisor_hostname', 'hypervisor_type',
                 'hypervisor_version', 'id', 'local_gb',
                 'local_gb_used', 'memory_mb', 'memory_mb_used',
                 'running_vms', 'service', 'vcpus', 'vcpus_used']
}

list_hypervisors_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'hypervisors': {
                'type': 'array',
                'items': hypervisor_detail
            }
        },
        'additionalProperties': False,
        'required': ['hypervisors']
    }
}

get_hypervisor = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'hypervisor': hypervisor_detail
        },
        'additionalProperties': False,
        'required': ['hypervisor']
    }
}

list_search_hypervisors = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'hypervisors': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'status': {'type': 'string'},
                        'state': {'type': 'string'},
                        'id': {'type': ['integer', 'string']},
                        'hypervisor_hostname': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    # NOTE: When loading os-hypervisor-status extension,
                    # a response contains status and state. So these params
                    # should not be required.
                    'required': ['id', 'hypervisor_hostname']
                }
            }
        },
        'additionalProperties': False,
        'required': ['hypervisors']
    }
}

get_hypervisor_uptime = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'hypervisor': {
                'type': 'object',
                'properties': {
                    'status': {'type': 'string'},
                    'state': {'type': 'string'},
                    'id': {'type': ['integer', 'string']},
                    'hypervisor_hostname': {'type': 'string'},
                    'uptime': {'type': 'string'}
                },
                'additionalProperties': False,
                # NOTE: When loading os-hypervisor-status extension,
                # a response contains status and state. So these params
                # should not be required.
                'required': ['id', 'hypervisor_hostname', 'uptime']
            }
        },
        'additionalProperties': False,
        'required': ['hypervisor']
    }
}

get_hypervisors_servers = copy.deepcopy(list_search_hypervisors)
get_hypervisors_servers['response_body']['properties']['hypervisors']['items'][
    'properties']['servers'] = {
        'type': 'array',
        'items': {
            'type': 'object',
            'properties': {
                'uuid': {'type': 'string'},
                'name': {'type': 'string'}
            },
            'additionalProperties': False,
        }
    }
# In V2 API, if there is no servers (VM) on the Hypervisor host then 'servers'
# attribute will not be present in response body So it is not 'required'.



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute.volumes import test_attach_volume
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class AttachVolumeNegativeTest(test_attach_volume.BaseAttachVolumeTest):
    """Negative tests of volume attaching"""

    @decorators.attr(type=['negative'])
    @decorators.related_bug('1630783', status_code=500)
    @decorators.idempotent_id('a313b5cd-fbd0-49cc-94de-870e99f763c7')
    def test_delete_attached_volume(self):
        """Test deleting attachemd volume should fail"""
        server, validation_resources = self._create_server()
        volume = self.create_volume()
        self.attach_volume(server, volume)

        self.assertRaises(lib_exc.BadRequest,
                          self.delete_volume, volume['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('aab919e2-d992-4cbb-a4ed-745c2475398c')
    def test_attach_attached_volume_to_same_server(self):
        """Test attaching attached volume to same server should fail

        Test attaching the same volume to the same instance once
        it's already attached. The nova/cinder validation for this differs
        depending on whether or not cinder v3.27 is being used to attach
        the volume to the instance.
        """
        server, validation_resources = self._create_server()
        volume = self.create_volume()

        self.attach_volume(server, volume)

        self.assertRaises(lib_exc.BadRequest,
                          self.attach_volume, server, volume)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ee37a796-2afb-11e7-bc0f-fa163e65f5ce')
    def test_attach_attached_volume_to_different_server(self):
        """Test attaching attached volume to different server should fail"""
        server1, validation_resources = self._create_server()
        volume = self.create_volume()

        self.attach_volume(server1, volume)

        # Create server2 and attach in-use volume
        server2, validation_resources = self._create_server()
        self.assertRaises(lib_exc.BadRequest,
                          self.attach_volume, server2, volume)



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class VolumePoolsAdminTestsJSON(base.BaseVolumeAdminTest):
    """Test getting volume pools by admin users"""

    def _assert_pools(self, with_detail=False):
        cinder_pools = self.admin_scheduler_stats_client.list_pools(
            detail=with_detail)['pools']
        self.assertNotEmpty(cinder_pools, "no cinder pools listed.")
        self.assertIn('name', cinder_pools[0])
        if with_detail:
            self.assertIn(CONF.volume.vendor_name,
                          [pool['capabilities']['vendor_name']
                           for pool in cinder_pools])

    @decorators.idempotent_id('0248a46c-e226-4933-be10-ad6fca8227e7')
    def test_get_pools_without_details(self):
        """Test getting volume pools without detail"""
        self._assert_pools()

    @decorators.idempotent_id('d4bb61f7-762d-4437-b8a4-5785759a0ced')
    def test_get_pools_with_details(self):
        """Test getting volume pools with detail"""
        self._assert_pools(with_detail=True)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from tempest.api.object_storage import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ObjectExpiryTest(base.BaseObjectTest):
    """Test object expiry"""

    @classmethod
    def resource_setup(cls):
        super(ObjectExpiryTest, cls).resource_setup()
        cls.container_name = cls.create_container()

    def setUp(self):
        super(ObjectExpiryTest, self).setUp()
        # create object
        self.object_name, _ = self.create_object(
            self.container_name)

    @classmethod
    def resource_cleanup(cls):
        cls.delete_containers()
        super(ObjectExpiryTest, cls).resource_cleanup()

    def _test_object_expiry(self, metadata):
        # update object metadata
        resp, _ = \
            self.object_client.create_or_update_object_metadata(
                self.container_name,
                self.object_name,
                headers=metadata)
        # verify object metadata
        resp, _ = \
            self.object_client.list_object_metadata(self.container_name,
                                                    self.object_name)
        self.assertHeaders(resp, 'Object', 'HEAD')
        self.assertIn('x-delete-at', resp)
        # we want to ensure that we will sleep long enough for things to
        # actually expire, so figure out how many secs in the future that is.
        sleepy_time = int(resp['x-delete-at']) - int(time.time())
        sleepy_time = sleepy_time if sleepy_time > 0 else 0
        resp, _ = self.object_client.get_object(self.container_name,
                                                self.object_name)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertIn('x-delete-at', resp)

        # add several seconds for safety.
        time.sleep(sleepy_time)

        # Checking whether object still exists for several seconds:
        # sometimes object is not deleted immediately, so we are making
        # get calls for an approximately 1 minute in a total. Get calls
        # can take 3s each sometimes so we are making the requests in
        # exponential periodicity
        for i in range(1, 6):
            time.sleep(2 ** i)
            try:
                self.object_client.get_object(self.container_name,
                                              self.object_name)
            except lib_exc.NotFound:
                break

        # object should not be there anymore
        self.assertRaises(lib_exc.NotFound,
                          self.object_client.get_object,
                          self.container_name,
                          self.object_name)

    @decorators.idempotent_id('fb024a42-37f3-4ba5-9684-4f40a7910b41')
    def test_get_object_after_expiry_time(self):
        """Test object is expired after x-delete-after time"""
        # the 10s is important, because the get calls can take 3s each
        # some times
        metadata = {'X-Delete-After': '10'}
        self._test_object_expiry(metadata)

    @decorators.idempotent_id('e592f18d-679c-48fe-9e36-4be5f47102c5')
    def test_get_object_at_expiry_time(self):
        """Test object is expired at x-delete-at time"""
        metadata = {'X-Delete-At': str(int(time.time()) + 10)}
        self._test_object_expiry(metadata)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class ServerDiskConfigTestJSON(base.BaseV2ComputeTest):
    """Test disk config option of server"""

    create_default_network = True

    @classmethod
    def skip_checks(cls):
        super(ServerDiskConfigTestJSON, cls).skip_checks()
        if not CONF.compute_feature_enabled.disk_config:
            msg = "DiskConfig extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(ServerDiskConfigTestJSON, cls).setup_clients()
        cls.client = cls.os_primary.servers_client

    def _update_server_with_disk_config(self, server_id, disk_config):
        server = self.client.show_server(server_id)['server']
        if disk_config != server['OS-DCF:diskConfig']:
            server = self.client.update_server(
                server_id, disk_config=disk_config)['server']
            waiters.wait_for_server_status(self.client, server['id'], 'ACTIVE')
            server = self.client.show_server(server['id'])['server']
            self.assertEqual(disk_config, server['OS-DCF:diskConfig'])

    @decorators.idempotent_id('bef56b09-2e8c-4883-a370-4950812f430e')
    def test_rebuild_server_with_manual_disk_config(self):
        """A server should be rebuilt using the manual disk config option"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.client.delete_server, server['id'])
        self._update_server_with_disk_config(server['id'],
                                             disk_config='AUTO')

        server = self.client.rebuild_server(server['id'],
                                            self.image_ref_alt,
                                            disk_config='MANUAL')['server']

        # Wait for the server to become active
        waiters.wait_for_server_status(self.client, server['id'], 'ACTIVE')

        # Verify the specified attributes are set correctly
        server = self.client.show_server(server['id'])['server']
        self.assertEqual('MANUAL', server['OS-DCF:diskConfig'])

    @decorators.idempotent_id('9c9fae77-4feb-402f-8450-bf1c8b609713')
    def test_rebuild_server_with_auto_disk_config(self):
        """A server should be rebuilt using the auto disk config option"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.client.delete_server, server['id'])
        self._update_server_with_disk_config(server['id'],
                                             disk_config='MANUAL')

        server = self.client.rebuild_server(server['id'],
                                            self.image_ref_alt,
                                            disk_config='AUTO')['server']

        # Wait for the server to become active
        waiters.wait_for_server_status(self.client, server['id'], 'ACTIVE')

        # Verify the specified attributes are set correctly
        server = self.client.show_server(server['id'])['server']
        self.assertEqual('AUTO', server['OS-DCF:diskConfig'])

    @decorators.idempotent_id('414e7e93-45b5-44bc-8e03-55159c6bfc97')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    def test_resize_server_from_manual_to_auto(self):
        """A server should be resized from manual to auto disk config"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.client.delete_server, server['id'])
        self._update_server_with_disk_config(server['id'],
                                             disk_config='MANUAL')
        # Resize with auto option
        self.resize_server(server['id'], self.flavor_ref_alt,
                           disk_config='AUTO')

        server = self.client.show_server(server['id'])['server']
        self.assertEqual('AUTO', server['OS-DCF:diskConfig'])

    @decorators.idempotent_id('693d16f3-556c-489a-8bac-3d0ca2490bad')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize not available.')
    def test_resize_server_from_auto_to_manual(self):
        """A server should be resized from auto to manual disk config"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.client.delete_server, server['id'])
        self._update_server_with_disk_config(server['id'],
                                             disk_config='AUTO')
        # Resize with manual option
        self.resize_server(server['id'], self.flavor_ref_alt,
                           disk_config='MANUAL')

        server = self.client.show_server(server['id'])['server']
        self.assertEqual('MANUAL', server['OS-DCF:diskConfig'])

    @decorators.idempotent_id('5ef18867-358d-4de9-b3c9-94d4ba35742f')
    def test_update_server_from_auto_to_manual(self):
        """A server should be updated from auto to manual disk config"""
        server = self.create_test_server(wait_until='ACTIVE')
        self.addCleanup(self.client.delete_server, server['id'])
        self._update_server_with_disk_config(server['id'],
                                             disk_config='AUTO')

        # Update the disk_config attribute to manual
        server = self.client.update_server(server['id'],
                                           disk_config='MANUAL')['server']
        waiters.wait_for_server_status(self.client, server['id'], 'ACTIVE')

        # Verify the disk_config attribute is set correctly
        server = self.client.show_server(server['id'])['server']
        self.assertEqual('MANUAL', server['OS-DCF:diskConfig'])



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from tempest.api.image import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class BasicOperationsImagesAdminTest(base.BaseV2ImageAdminTest):
    """"Test image operations about image owner"""

    @decorators.related_bug('1420008')
    @decorators.idempotent_id('646a6eaa-135f-4493-a0af-12583021224e')
    def test_create_image_owner_param(self):
        """Test creating image with specified owner"""
        # NOTE: Create image with owner different from tenant owner by
        # using "owner" parameter requires an admin privileges.
        random_id = data_utils.rand_uuid_hex()
        image = self.admin_client.create_image(
            container_format='bare', disk_format='raw', owner=random_id)
        self.addCleanup(self.admin_client.delete_image, image['id'])
        image_info = self.admin_client.show_image(image['id'])
        self.assertEqual(random_id, image_info['owner'])

    @decorators.related_bug('1420008')
    @decorators.idempotent_id('525ba546-10ef-4aad-bba1-1858095ce553')
    def test_update_image_owner_param(self):
        """Test updating image owner"""
        random_id_1 = data_utils.rand_uuid_hex()
        image = self.admin_client.create_image(
            container_format='bare', disk_format='raw', owner=random_id_1)
        self.addCleanup(self.admin_client.delete_image, image['id'])
        created_image_info = self.admin_client.show_image(image['id'])

        random_id_2 = data_utils.rand_uuid_hex()
        self.admin_client.update_image(
            image['id'], [dict(replace="/owner", value=random_id_2)])
        updated_image_info = self.admin_client.show_image(image['id'])

        self.assertEqual(random_id_2, updated_image_info['owner'])
        self.assertNotEqual(created_image_info['owner'],
                            updated_image_info['owner'])

    @decorators.idempotent_id('f6ab4aa0-035e-4664-9f2d-c57c6df50605')
    def test_list_public_image(self):
        """Test create image as admin and list public image as none admin"""
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Image')
        image = self.admin_client.create_image(
            name=name,
            container_format='bare',
            visibility='public',
            disk_format='raw')
        waiters.wait_for_image_status(self.admin_client, image['id'], 'queued')
        created_image = self.admin_client.show_image(image['id'])
        self.assertEqual(image['id'], created_image['id'])
        self.addCleanup(self.admin_client.delete_image, image['id'])

        images_list = self.client.list_images()['images']
        fetched_images_id = [img['id'] for img in images_list]
        self.assertIn(image['id'], fetched_images_id)


class ImportCopyImagesTest(base.BaseV2ImageAdminTest):
    """Test the import copy-image operations"""

    @classmethod
    def skip_checks(cls):
        super(ImportCopyImagesTest, cls).skip_checks()
        if not CONF.image_feature_enabled.import_image:
            skip_msg = (
                "%s skipped as image import is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @decorators.idempotent_id('9b3b644e-03d1-11eb-a036-fa163e2eaf49')
    def test_image_copy_image_import(self):
        """Test 'copy-image' import functionalities

        Create image, import image with copy-image method and
        verify that import succeeded.
        """
        available_stores = self.get_available_stores()
        available_import_methods = self.client.info_import()[
            'import-methods']['value']
        # NOTE(gmann): Skip if copy-image import method and multistore
        # are not available.
        if ('copy-image' not in available_import_methods or
            not available_stores):
            raise self.skipException('Either copy-image import method or '
                                     'multistore is not available')
        uuid = data_utils.rand_uuid()
        image_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='copy-image')
        container_format = CONF.image.container_formats[0]
        disk_format = 'raw'
        image = self.create_image(name=image_name,
                                  container_format=container_format,
                                  disk_format=disk_format,
                                  visibility='private',
                                  ramdisk_id=uuid)
        self.assertEqual('queued', image['status'])

        file_content = data_utils.random_bytes()
        image_file = io.BytesIO(file_content)
        self.client.store_image_file(image['id'], image_file)

        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual(len(file_content), body.get('size'))
        self.assertEqual('active', body['status'])

        # Copy image to all the stores. In case of all_stores request
        # glance will skip the stores where image is already available.
        self.admin_client.image_import(image['id'], method='copy-image',
                                       all_stores=True,
                                       all_stores_must_succeed=False)

        # Wait for copy to finished on all stores.
        failed_stores = waiters.wait_for_image_copied_to_stores(
            self.client, image['id'])
        # Assert if copy is failed on any store.
        self.assertEqual(0, len(failed_stores),
                         "Failed to copy the following stores: %s" %
                         str(failed_stores))


class ImageLocationsAdminTest(base.BaseV2ImageAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ImageLocationsAdminTest, cls).skip_checks()
        if not CONF.image_feature_enabled.manage_locations:
            skip_msg = (
                "%s skipped as show_multiple_locations is not available" % (
                    cls.__name__))
            raise cls.skipException(skip_msg)

    @decorators.idempotent_id('8a648de4-b745-4c28-a7b5-20de1c3da4d2')
    def test_delete_locations(self):
        image = self.check_set_multiple_locations()
        expected_remaining_loc = image['locations'][1]

        self.admin_client.update_image(image['id'], [
            dict(remove='/locations/0')])

        # The image should now have only the one location we did not delete
        image = self.client.show_image(image['id'])
        self.assertEqual(1, len(image['locations']),
                         'Image should have one location but has %i' % (
                         len(image['locations'])))
        self.assertEqual(expected_remaining_loc['url'],
                         image['locations'][0]['url'])

        # The direct_url should now be the last remaining location
        if 'direct_url' in image:
            self.assertEqual(image['direct_url'], image['locations'][0]['url'])

        # Removing the last location should be disallowed
        self.assertRaises(lib_exc.Forbidden,
                          self.admin_client.update_image, image['id'], [
                              dict(remove='/locations/0')])


class MultiStoresImagesTest(base.BaseV2ImageAdminTest, base.BaseV2ImageTest):
    """Test importing and deleting image in multiple stores"""
    @classmethod
    def skip_checks(cls):
        super(MultiStoresImagesTest, cls).skip_checks()
        if not CONF.image_feature_enabled.import_image:
            skip_msg = (
                "%s skipped as image import is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(MultiStoresImagesTest, cls).resource_setup()
        cls.available_import_methods = \
            cls.client.info_import()['import-methods']['value']
        if not cls.available_import_methods:
            raise cls.skipException('Server does not support '
                                    'any import method')

        # NOTE(pdeore): Skip if glance-direct import method and mutlistore
        # are not enabled/configured, or only one store is configured in
        # multiple stores setup.
        cls.available_stores = cls.get_available_stores()
        if ('glance-direct' not in cls.available_import_methods or
                not len(cls.available_stores) > 1):
            raise cls.skipException(
                'Either glance-direct import method not present in %s or '
                'None or only one store is '
                'configured %s' % (cls.available_import_methods,
                                   cls.available_stores))

    @decorators.idempotent_id('1ecec683-41d4-4470-a0df-54969ec74514')
    def test_delete_image_from_specific_store(self):
        """Test delete image from specific store"""
        # Import image to available stores
        image, stores = self.create_and_stage_image(all_stores=True)
        self.client.image_import(image['id'],
                                 method='glance-direct',
                                 all_stores=True)
        self.addCleanup(self.admin_client.delete_image, image['id'])
        waiters.wait_for_image_imported_to_stores(
            self.client,
            image['id'], stores)
        observed_image = self.client.show_image(image['id'])

        # Image will be deleted from first store
        first_image_store_deleted = (observed_image['stores'].split(","))[0]
        self.admin_client.delete_image_from_store(
            observed_image['id'], first_image_store_deleted)
        waiters.wait_for_image_deleted_from_store(
            self.admin_client,
            observed_image,
            stores,
            first_image_store_deleted)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import fixtures

from tempest.lib.services.compute import versions_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestVersionsClient(base.BaseServiceTest):

    FAKE_INIT_VERSION = {
        "version": {
            "id": "v2.1",
            "links": [
                {
                    "href": "http://openstack.example.com/v2.1/",
                    "rel": "self"
                },
                {
                    "href": "http://docs.openstack.org/",
                    "rel": "describedby",
                    "type": "text/html"
                }
            ],
            "status": "CURRENT",
            "updated": "2013-07-23T11:33:21Z",
            "version": "2.1",
            "min_version": "2.1"
            }
        }

    FAKE_VERSIONS_INFO = {
        "versions": [FAKE_INIT_VERSION["version"]]
        }

    FAKE_VERSION_INFO = copy.deepcopy(FAKE_INIT_VERSION)

    FAKE_VERSION_INFO["version"]["media-types"] = [
        {
            "base": "application/json",
            "type": "application/vnd.openstack.compute+json;version=2.1"
        }
        ]

    def setUp(self):
        super(TestVersionsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.versions_client = (
            versions_client.VersionsClient
            (fake_auth, 'compute', 'regionOne'))

    def _test_versions_client(self, bytes_body=False):
        self.check_service_client_function(
            self.versions_client.list_versions,
            'tempest.lib.common.rest_client.RestClient.raw_request',
            self.FAKE_VERSIONS_INFO,
            bytes_body,
            200)

    def _test_get_version_by_url(self, bytes_body=False):
        self.useFixture(fixtures.MockPatch(
            "tempest.lib.common.rest_client.RestClient.token",
            return_value="Dummy Token"))
        params = {"version_url": self.versions_client._get_base_version_url()}
        self.check_service_client_function(
            self.versions_client.get_version_by_url,
            'tempest.lib.common.rest_client.RestClient.raw_request',
            self.FAKE_VERSION_INFO,
            bytes_body,
            200, **params)

    def test_list_versions_client_with_str_body(self):
        self._test_versions_client()

    def test_list_versions_client_with_bytes_body(self):
        self._test_versions_client(bytes_body=True)

    def test_get_version_by_url_with_str_body(self):
        self._test_get_version_by_url()

    def test_get_version_by_url_with_bytes_body(self):
        self._test_get_version_by_url(bytes_body=True)

    def _test_get_base_version_url(self, url, expected_base_url):
        auth = fake_auth_provider.FakeAuthProvider(fake_base_url=url)
        client = versions_client.VersionsClient(auth, 'compute', 'regionOne')
        self.assertEqual(expected_base_url, client._get_base_version_url())

    def test_get_base_version_url(self):
        self._test_get_base_version_url('https://bar.org/v2/123',
                                        'https://bar.org/')
        self._test_get_base_version_url('https://bar.org/v2.1/123',
                                        'https://bar.org/')
        self._test_get_base_version_url('https://bar.org/v2.15/123',
                                        'https://bar.org/')
        self._test_get_base_version_url('https://bar.org/v22.2/123',
                                        'https://bar.org/')
        self._test_get_base_version_url('https://bar.org/v22/123',
                                        'https://bar.org/')

    def test_get_base_version_url_app_name(self):
        self._test_get_base_version_url('https://bar.org/compute/v2/123',
                                        'https://bar.org/compute/')
        self._test_get_base_version_url('https://bar.org/compute/v2.1/123',
                                        'https://bar.org/compute/')
        self._test_get_base_version_url('https://bar.org/compute/v2.15/123',
                                        'https://bar.org/compute/')
        self._test_get_base_version_url('https://bar.org/compute/v22.2/123',
                                        'https://bar.org/compute/')
        self._test_get_base_version_url('https://bar.org/compute/v22/123',
                                        'https://bar.org/compute/')

    def test_get_base_version_url_double_slash(self):
        self._test_get_base_version_url('https://bar.org//v2/123',
                                        'https://bar.org/')
        self._test_get_base_version_url('https://bar.org//v2.1/123',
                                        'https://bar.org/')
        self._test_get_base_version_url('https://bar.org/compute//v2/123',
                                        'https://bar.org/compute/')
        self._test_get_base_version_url('https://bar.org/compute//v2.1/123',
                                        'https://bar.org/compute/')



# Copyright (c) 2015 Deutsche Telekom AG
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services import clients
from tempest.test_discover import plugins
from tempest.tests import base
from tempest.tests import fake_tempest_plugin as fake_plugin
from tempest.tests.lib.services import registry_fixture


class TestPluginDiscovery(base.TestCase):

    def setUp(self):
        super(TestPluginDiscovery, self).setUp()
        # Make sure we leave the registry clean
        self.useFixture(registry_fixture.RegistryFixture())

    def test_load_tests_with_one_plugin(self):
        # we can't mock stevedore since it's a singleton and already executed
        # during test discovery. So basically this test covers the plugin loop
        # and the abstract plugin interface.
        manager = plugins.TempestTestPluginManager()
        fake_obj = fake_plugin.FakeStevedoreObj()
        manager.ext_plugins = [fake_obj]
        result = manager.get_plugin_load_tests_tuple()

        self.assertEqual(fake_plugin.FakePlugin.expected_load_test,
                         result[fake_obj.name])

    def test_load_tests_with_two_plugins(self):
        manager = plugins.TempestTestPluginManager()
        obj1 = fake_plugin.FakeStevedoreObj('fake01')
        obj2 = fake_plugin.FakeStevedoreObj('fake02')
        manager.ext_plugins = [obj1, obj2]
        result = manager.get_plugin_load_tests_tuple()

        self.assertEqual(fake_plugin.FakePlugin.expected_load_test,
                         result['fake01'])
        self.assertEqual(fake_plugin.FakePlugin.expected_load_test,
                         result['fake02'])

    def test__register_service_clients_with_one_plugin(self):
        registry = clients.ClientsRegistry()
        manager = plugins.TempestTestPluginManager()
        fake_obj = fake_plugin.FakeStevedoreObj()
        manager.ext_plugins = [fake_obj]
        manager._register_service_clients()
        expected_result = fake_plugin.FakePlugin.expected_service_clients
        registered_clients = registry.get_service_clients()
        self.assertIn(fake_obj.name, registered_clients)
        self.assertEqual(expected_result, registered_clients[fake_obj.name])

    def test__get_service_clients_with_two_plugins(self):
        registry = clients.ClientsRegistry()
        manager = plugins.TempestTestPluginManager()
        obj1 = fake_plugin.FakeStevedoreObj('fake01')
        obj2 = fake_plugin.FakeStevedoreObj('fake02')
        manager.ext_plugins = [obj1, obj2]
        manager._register_service_clients()
        expected_result = fake_plugin.FakePlugin.expected_service_clients
        registered_clients = registry.get_service_clients()
        self.assertIn('fake01', registered_clients)
        self.assertIn('fake02', registered_clients)
        self.assertEqual(expected_result, registered_clients['fake01'])
        self.assertEqual(expected_result, registered_clients['fake02'])

    def test__register_service_clients_one_plugin_no_service_clients(self):
        registry = clients.ClientsRegistry()
        manager = plugins.TempestTestPluginManager()
        fake_obj = fake_plugin.FakeStevedoreObjNoServiceClients()
        manager.ext_plugins = [fake_obj]
        manager._register_service_clients()
        registered_clients = registry.get_service_clients()
        self.assertNotIn(fake_obj.name, registered_clients)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class IdentityClient(rest_client.RestClient):
    api_version = "v3"

    def show_api_description(self):
        """Retrieves info about the v3 Identity API"""
        url = ''
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_token(self, resp_token):
        """Get token details."""
        headers = {'X-Subject-Token': resp_token}
        resp, body = self.get("auth/tokens", headers=headers)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_token(self, resp_token):
        """Deletes token."""
        headers = {'X-Subject-Token': resp_token}
        resp, body = self.delete("auth/tokens", headers=headers)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def check_token_existence(self, resp_token):
        """Validates a token."""
        headers = {'X-Subject-Token': resp_token}
        resp, body = self.head("auth/tokens", headers=headers)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_auth_projects(self):
        """Get available project scopes."""
        resp, body = self.get("auth/projects")
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_auth_domains(self):
        """Get available domain scopes."""
        resp, body = self.get("auth/domains")
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)
